#!/usr/bin/python3
# -*- coding: utf-8 -*-

# main.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import sys

from slpkg.clean import clean_tmp
from slpkg.load import Regex
from slpkg.desc import PkgDesc
from slpkg.messages import Msg
from slpkg.auto_pkg import Auto
from slpkg.config import Config
from slpkg.checks import Updates
from slpkg.repoinfo import RepoInfo
from slpkg.repolist import RepoList
from slpkg.repositories import Repo
from slpkg.blacklist import BlackList
from slpkg.version import prog_version
from slpkg.health import PackageHealth
from slpkg.new_config import NewConfig
from slpkg.tracking import TrackingDeps
from slpkg.repoenable import RepoEnable
from slpkg.pkg_find import FindFromRepos
from slpkg.arguments import options, usage
from slpkg.slpkg_update import it_self_update
from slpkg.status_deps import DependenciesStatus

from slpkg.init import (
    Update,
    Upgrade,
    check_exists_repositories
)
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.manager import PackageManager

from slpkg.sbo.queue import QueuePkgs
from slpkg.sbo.check import sbo_upgrade
from slpkg.sbo.network import SBoNetwork
from slpkg.sbo.autobuild import AutoBuild
from slpkg.sbo.slackbuild import SBoInstall

from slpkg.slack.patches import Patches
from slpkg.binary.check import pkg_upgrade
from slpkg.binary.install import BinaryInstall


class ArgParse(BlackList):

    def __init__(self, args):
        super().__init__()
        self.args = args
        self.meta = _meta_
        self.msg = Msg()
        self.commands = [
            "update",
            "upgrade",
            "repo-add",
            "repo-remove",
            "repo-enable",
            "repo-list",
            "repo-info",
            "update-slpkg",
            "health",
            "deps-status",
            "new-config",
            "clean"
        ]

        options = [
            "-c", "--check",
            "-l", "--list",
            "-c", "--check",
            "-s", "--sync",
            "-t", "--tracking",
            "-p", "--desc",
            "-F", "--FIND",
            "-f", "--find"
        ]

        # checking if repositories exists
        enabled_repos = _meta_.repositories
        if len(self.args) > 1:
            repo = self.args[1]
            check = check_exists_repositories(repo)
        if (len(self.args) > 1 and self.args[0] in options
                and check is False and repo in enabled_repos):
            print("\n  Please update the packages lists. Run 'slpkg update'.\n"
                  "  This command should be used to synchronize the packages\n"
                  "  lists from the repositories that are enabled.\n")
            raise SystemExit(1)

    def help_version(self):
        """Help and version info
        """
        if (len(self.args) == 1 and self.args[0] in ["-h", "--help"] and
                self.args[1:] == []):
            options()
        elif (len(self.args) == 1 and self.args[0] in ["-v", "--version"] and
                self.args[1:] == []):
            prog_version()
        else:
            usage("", 1)

    def command_update(self):
        """Update package lists repositories
        """
        update = Update()
        if len(self.args) == 1 and self.args[0] == "update":
            update.run(repos="")
        elif (len(self.args) == 2 and self.args[0] == "update" and
                self.args[1].startswith("--repos=")):
            repos = self.args[1].split("=")[-1].split(",")
            for rp in repos:
                if rp not in self.meta.repositories:
                    usage(rp, 1)
            update.run(repos)
        else:
            usage("", 1)

    def command_upgrade(self):
        """Recreate repositories package lists
        """
        upgrade = Upgrade()
        if len(self.args) == 1 and self.args[0] == "upgrade":
            upgrade.run(repos="")
        elif (len(self.args) == 2 and self.args[0] == "upgrade" and
                self.args[1].startswith("--repos=")):
            repos = self.args[1].split("=")[-1].split(",")
            for rp in repos:
                if rp not in self.meta.repositories:
                    usage(rp, 1)
            upgrade.run(repos)
        else:
            usage("", 1)

    def command_update_slpkg(self):
        """Slpkg it self update
        """
        if len(self.args) == 2 and self.args[0] == "update-slpkg":
            it_self_update()
        else:
            usage("", 1)

    def command_repo_enable(self):
        """Repositories enable/disable
        """
        self.if_checklist()
        if len(self.args) == 1 and self.args[0] == "repo-enable":
            RepoEnable().choose()
        else:
            usage("", 1)

    def command_repo_list(self):
        """Repositories list
        """
        if len(self.args) == 1 and self.args[0] == "repo-list":
            RepoList().repos()
        else:
            usage("", 1)

    def command_repo_add(self):
        """Add custom repositories
        """
        if len(self.args) == 3 and self.args[0] == "repo-add":
            Repo().add(self.args[1], self.args[2])
        else:
            usage("", 1)

    def command_repo_remove(self):
        """Remove custom repositories
        """
        if len(self.args) == 2 and self.args[0] == "repo-remove":
            Repo().remove(self.args[1])
        else:
            usage("", 1)

    def command_repo_info(self):
        """Repositories informations
        """
        if (len(self.args) == 2 and self.args[0] == "repo-info" and
                self.args[1] in RepoList().all_repos):
            del RepoList().all_repos
            RepoInfo().view(self.args[1])
        elif (len(self.args) > 1 and self.args[0] == "repo-info" and
                self.args[1] not in RepoList().all_repos):
            usage(self.args[1], 1)
        else:
            usage("", 1)

    def command_health(self):
        """Check package health
        """
        if len(self.args) == 1 and self.args[0] == "health":
            PackageHealth(mode="").test()
        elif (len(self.args) == 2 and self.args[0] == "health" and
                self.args[1] == "--silent"):
            PackageHealth(mode=self.args[1]).test()
        else:
            usage("", 1)

    def command_deps_status(self):
        """Print dependencies status
        """
        image = ""
        for arg in self.args:
            if arg.startswith("--graph="):
                image = arg.split("=")[1]
        if len(self.args) == 1 and self.args[0] == "deps-status":
            DependenciesStatus(image).show()
        elif len(self.args) == 2 and self.args[0] == "deps-status" and image:
            DependenciesStatus(image).show()
        elif (len(self.args) == 2 and self.args[0] == "deps-status" and
                "--tree" in self.args):
            DependenciesStatus(image).tree()
        elif (len(self.args) == 3 and self.args[0] == "deps-status" and
                "--tree" in self.args and image):
            DependenciesStatus(image).tree()
        else:
            usage("", 1)

    def command_new_config(self):
        """Manage .new configuration files
        """
        if len(self.args) == 1 and self.args[0] == "new-config":
            NewConfig().run()
        else:
            usage("", 1)

    def command_clean_tmp(self):
        """Clean all downloaded packages and sources"""
        if len(self.args) == 1 and self.args[0] == "clean-tmp":
            clean_tmp()
        else:
            usage("", 1)

    def auto_build(self):
        """Auto built tool
        """
        options = [
            "-a",
            "--autobuild"
        ]
        if len(self.args) >= 3 and self.args[0] in options:
            AutoBuild(self.args[1], self.args[2:], self.meta.path).run()
        else:
            usage("", 1)

    def pkg_list(self):
        """List of packages by repository
        """
        options = [
            "-l",
            "--list"
        ]
        flag = ["--index", "--installed", "--name"]
        name = INDEX = installed = False
        for arg in self.args[2:]:
            if flag[0] == arg:
                INDEX = True
            if flag[1] in arg:
                installed = True
            if flag[2] == arg:
                name = True
            if arg not in flag:
                usage("", 1)
        if (len(self.args) > 1 and len(self.args) <= 5 and
                self.args[0] in options):
            if self.args[1] in self.meta.repositories:
                PackageManager(binary=None).package_list(self.args[1],
                                                         name,
                                                         INDEX,
                                                         installed)
            else:
                usage(self.args[1], 1)
        else:
            usage("", 1)

    def pkg_upgrade(self):
        """Check and upgrade packages by repository
        """
        options = [
            "-c",
            "--check"
        ]
        flags = [
            "--upgrade",
            "--skip=",
            "--resolve-off",
            "--checklist",
            "--rebuild"
        ]
        flag, skip = self.__pkg_upgrade_flags(flags)

        # Remove --checklist flag from args so that works with
        # both conditions
        # if flags[3] in self.args:
        #    self.args.remove(flags[3])

        if (len(self.args) >= 3 and self.args[0] in options and
                self.args[2] == flags[0] and
                self.args[1] in self.meta.repositories):
            if self.args[1] not in ["slack", "sbo"]:
                BinaryInstall(pkg_upgrade(self.args[1], skip, flag),
                              self.args[1], flag).start(is_upgrade=True)
            elif self.args[1] == "slack":
                Patches(skip, flag).start()
            elif self.args[1] == "sbo":
                SBoInstall(sbo_upgrade(skip, flag), flag).start(
                    is_upgrade=True)
            else:
                usage(self.args[1], 1)
        elif len(self.args) == 1 and self.args[0] in options:
            Updates(repo="").ALL()
        elif len(self.args) == 2 and self.args[0] in options:
            Updates(self.args[1]).run()
        elif (len(self.args) >= 2 and self.args[0] in options and
              self.args[1] not in self.meta.repositories):
            usage(self.args[1], 1)
        else:
            usage("", 1)

    def __pkg_upgrade_flags(self, flags):
        """Manage flags for package upgrade option
        """
        # Check for typos or unssuported flags
        for arg in self.args[2:]:
            if arg not in flags:
                usage("", 1)

        flag, skip = [], ""
        if flags[0] in self.args:
            for arg in self.args:
                if arg.startswith(flags[1]):
                    skip = Regex(arg.split("=")[1]).get()
                    self.args.remove(arg)
                if arg in flags:
                    flag.append(arg)
                    if arg not in self.args:
                        self.args.remove(arg)
        if "--checklist" in flag:
            self.if_checklist()
        return flag, skip

    def pkg_install(self):
        """Install packages by repository
        """
        flag = []
        options = [
            "-s",
            "--sync"
        ]
        additional_options = [
            "--resolve-off",
            "--download-only",
            "--directory-prefix=",
            "--case-ins",
            "--rebuild",
            "--reinstall",
            "--patches"
        ]
        for arg in self.args:
            if arg.startswith(additional_options[2]):
                flag.append(arg)
                arg = ""
            if arg in additional_options:
                flag.append(arg)
        # clean from flags
        for ar in flag:
            if ar in self.args:
                self.args.remove(ar)
        if len(self.args) >= 3 and self.args[0] in options:
            if (self.args[1] in self.meta.repositories and
                    self.args[1] not in ["sbo"]):
                BinaryInstall(self.args[2:], self.args[1], flag).start(
                    is_upgrade=False)
            elif (self.args[1] == "sbo" and
                    self.args[1] in self.meta.repositories):
                SBoInstall(self.args[2:], flag).start(is_upgrade=False)
            else:
                usage(self.args[1], 1)
        else:
            usage("", 1)

    def pkg_tracking(self):
        """Tracking package dependencies
        """
        flag = []
        options = [
            "-t",
            "--tracking"
        ]
        additional_options = [
            "--check-deps",
            "--graph=",
            "--case-ins"
        ]
        for arg in self.args[2:]:
            if arg.startswith(additional_options[1]):
                flag.append(arg)
                self.args.remove(arg)
            if arg in additional_options:
                flag.append(arg)
        # clean additional options from args
        for f in flag:
            if f in self.args:
                self.args.remove(f)
        # print usage message if wrong additional option
        for arg in self.args:
            if arg.startswith("--"):
                if arg not in additional_options:
                    usage("", 1)
        if (len(self.args) >= 3 and len(self.args) <= 3 and
                self.args[0] in options and
                self.args[1] in self.meta.repositories):
            TrackingDeps(self.args[2], self.args[1], flag).run()
        elif (len(self.args) >= 2 and
                self.args[1] not in self.meta.repositories):
            usage(self.args[1], 1)
        else:
            usage("", 1)

    def sbo_network(self):
        """View slackbuilds packages
        """
        flag = []
        options = [
            "-n",
            "--network"
        ]
        additional_options = [
            "--checklist",
            "--case-ins"
        ]
        for add in additional_options:
            if add in self.args:
                flag.append(add)
                self.args.remove(add)
        if "--checklist" in flag:
            self.if_checklist()
        if (len(self.args) == 2 and self.args[0] in options and
                "sbo" in self.meta.repositories):
            SBoNetwork(self.args[1], flag).view()
        elif (len(self.args) == 1 and self.args[0] in options and
                "sbo" in self.meta.repositories and
                additional_options[0] in flag):
            SBoNetwork("", flag).view()
        else:
            usage("sbo", 1)

    def pkg_blacklist(self):
        """Manage blacklist packages
        """
        options = [
            "-b",
            "--blacklist"
        ]
        flag = [
            "--add",
            "--remove"
        ]
        command = ["list"]
        if (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == command[0]):
            self.black_listed()
        elif (len(self.args) > 2 and self.args[0] in options and
                flag[0] in self.args):
            self.args.remove(flag[0])
            self.black_add(self.args[1:])
        elif (len(self.args) == 2 and self.args[0] in options and
                flag[1] in self.args):
            self.args.remove(flag[1])
            self.black_remove(list(self.get_black()))
        elif (len(self.args) > 2 and self.args[0] in options and
                flag[1] in self.args):
            self.args.remove(flag[1])
            self.black_remove(self.args[1:])
        else:
            usage("", 1)

    def pkg_queue(self):
        """Manage packages in queue
        """
        queue = QueuePkgs()
        options = [
            "-q",
            "--queue"
        ]
        flag = [
            "--add",
            "--remove"
        ]
        command = [
            "list",
            "build",
            "install",
            "build-install"
        ]
        if (len(self.args) > 2 and self.args[0] in options and
                flag[0] in self.args):
            self.args.remove(flag[0])
            queue.add(self.args[1:])
        elif (len(self.args) == 2 and self.args[0] in options and
                flag[1] in self.args):
            self.args.remove(flag[1])
            queue.remove(queue.packages())
        elif (len(self.args) > 2 and self.args[0] in options and
                flag[1] in self.args):
            self.args.remove(flag[1])
            queue.remove(self.args[1:])
        elif (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == command[0]):
            queue.listed()
        elif (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == command[1]):
            queue.build()
        elif (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == command[2]):
            queue.install()
        elif (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == command[3]):
            queue.build()
            queue.install()
        else:
            usage("", 1)

    def bin_install(self):
        """Install Slackware binary packages
        """
        packages = self.args[1:]
        options = [
            "-i",
            "--installpkg"
        ]
        flag = ""
        flags = [
            "--warn",
            "--md5sum",
            "--root",
            "--infobox",
            "--menu",
            "--terse",
            "--ask",
            "--priority",
            "--tagfile"
        ]
        if len(self.args) > 1 and self.args[0] in options:
            if self.args[1] in flags:
                flag = self.args[1]
                packages = self.args[2:]
            PackageManager(packages).install(flag)
        else:
            usage("", 1)

    def bin_upgrade(self):
        """Install-upgrade Slackware binary packages
        """
        packages = self.args[1:]
        options = [
            "-u",
            "--upgradepkg"
        ]
        flag = ""
        flags = [
            "--dry-run",
            "--install-new",
            "--reinstall",
            "--verbose"
        ]
        if len(self.args) > 1 and self.args[0] in options:
            if self.args[1] in flags:
                flag = self.args[1]
                packages = self.args[2:]
            PackageManager(packages).upgrade(flag)
        else:
            usage("", 1)

    def bin_remove(self):
        """Remove Slackware packages
        """
        packages = self.args[1:]
        options = [
            "-r",
            "--removepkg"
        ]
        additional_options = [
            "--deps",
            "--check-deps",
            "--tag",
            "--checklist",
            "--third-party"
        ]
        flag, extra = "", []
        flags = [
            "-warn",
            "-preserve",
            "-copy",
            "-keep"
        ]
        # merge --check-deps and --deps options
        if (additional_options[1] in self.args and
                additional_options[0] not in self.args):
            self.args.append(additional_options[0])
        if len(self.args) > 1 and self.args[0] in options:
            for additional in additional_options:
                if additional in self.args:
                    extra.append(additional)
                    self.args.remove(additional)
                packages = self.args[1:]
            for fl in flags:
                if fl in self.args:
                    flag = self.args[1]
                    packages = self.args[2:]
            if "--checklist" in extra:
                self.if_checklist()
            if not packages:
                packages = [""]
            PackageManager(packages).remove(flag, extra)
        else:
            usage("", 1)

    def bin_find(self):
        """Find installed packages
        """
        flag = []
        options = [
            "-f",
            "--find"
        ]
        additional_options = [
            "--case-ins",
            "--third-party"
        ]
        for add in additional_options:
            if add in self.args:
                flag.append(add)
                self.args.remove(add)
        packages = self.args[1:]
        if not packages:
            packages = [""]
        if len(self.args) == 1 and self.args[0] in options:
            PackageManager(packages).find(flag)
        elif len(self.args) > 1 and self.args[0] in options:
            PackageManager(packages).find(flag)
        else:
            usage("", 1)

    def pkg_desc(self):
        """Print slack-desc by repository
        """
        options = [
            "-p",
            "--desc"
        ]
        flag = ["--color="]
        colors = [
            "red",
            "green",
            "yellow",
            "cyan",
            "grey"
        ]
        tag = ""
        for arg in self.args:
            if arg.startswith(flag[0]):
                tag = arg[len(flag[0]):]
                self.args.remove(arg)
                break
        if tag and tag not in colors:
            raise SystemExit(f"\nslpkg: Error: Available colors {colors}\n")
        if (len(self.args) == 3 and self.args[0] in options and
                self.args[1] in self.meta.repositories and tag in colors):
            PkgDesc(self.args[2], self.args[1], tag).view()
        elif (len(self.args) == 3 and self.args[0] in options and
                self.args[1] in self.meta.repositories):
            PkgDesc(self.args[2], self.args[1], paint="").view()
        elif (len(self.args) > 1 and self.args[0] in options and
                self.args[1] not in self.meta.repositories):
            usage(self.args[1], 1)
        else:
            usage("", 1)

    def pkg_find(self):
        """Find packages from all enabled repositories
        """
        flag = []
        options = [
            "-F",
            "--FIND"
        ]
        additional_options = ["--case-ins"]
        for arg in self.args:
            if arg in additional_options:
                flag.append(arg)
                self.args.remove(arg)
        packages = self.args[1:]
        if len(self.args) > 1 and self.args[0] in options:
            FindFromRepos().find(packages, flag)
        else:
            usage("", 1)

    def pkg_contents(self):
        """Print packages contents
        """
        packages = self.args[1:]
        options = [
            "-d",
            "--display"
        ]
        if len(self.args) > 1 and self.args[0] in options:
            PackageManager(packages).display()
        else:
            usage("", 1)

    def congiguration(self):
        """Manage slpkg configuration file
        """
        options = [
            "-g",
            "--config"
        ]
        command = [
            "print",
            "edit",
            "reset"
        ]
        conf = Config()
        if (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == command[1]):
            conf.edit()
        elif (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == (command[0])):
            conf.view()
        elif (len(self.args) == 2 and self.args[0] in options and
                self.args[1] == (command[2])):
            conf.reset()
        else:
            usage("", 1)

    def auto_detect(self, args):
        """Check for already Slackware binary packages exist
        """
        suffixes = [
            ".tgz",
            ".txz",
            ".tbz",
            ".tlz"
        ]
        if (not args[0].startswith("-") and args[0] not in self.commands and
                args[0].endswith(tuple(suffixes))):
            packages, not_found = [], []
            for pkg in args:
                if pkg.endswith(tuple(suffixes)):
                    if os.path.isfile(pkg):
                        packages.append(pkg)
                    else:
                        not_found.append(pkg)
            if packages:
                Auto(packages).select()
            if not_found:
                for ntf in not_found:
                    self.msg.pkg_not_found("", ntf, "Not installed", "")
            raise SystemExit(1)

    def if_checklist(self):
        try:
            from dialog import Dialog
        except ImportError:
            raise SystemExit("Require 'pythondialog': Install with 'slpkg "
                             "-s sbo python3-pythondialog'")


def main():

    args = sys.argv
    args.pop(0)

    argparse = ArgParse(args)

    if len(args) == 0:
        usage("", 1)

    argparse.auto_detect(args)

    if len(args) == 2 and args[0] == "update" and args[1] == "slpkg":
        args[0] = "update-slpkg"

    arguments = {
        "-h": argparse.help_version,
        "--help": argparse.help_version,
        "-v": argparse.help_version,
        "--version": argparse.help_version,
        "update": argparse.command_update,
        "upgrade": argparse.command_upgrade,
        "update-slpkg": argparse.command_update_slpkg,
        "repo-enable": argparse.command_repo_enable,
        "repo-list": argparse.command_repo_list,
        "repo-add": argparse.command_repo_add,
        "repo-remove": argparse.command_repo_remove,
        "repo-info": argparse.command_repo_info,
        "health": argparse.command_health,
        "deps-status": argparse.command_deps_status,
        "new-config": argparse.command_new_config,
        "clean-tmp": argparse.command_clean_tmp,
        "-a": argparse.auto_build,
        "--autobuild": argparse.auto_build,
        "-l": argparse.pkg_list,
        "--list": argparse.pkg_list,
        "-c": argparse.pkg_upgrade,
        "--check": argparse.pkg_upgrade,
        "-s": argparse.pkg_install,
        "--sync": argparse.pkg_install,
        "-t": argparse.pkg_tracking,
        "--tracking": argparse.pkg_tracking,
        "-n": argparse.sbo_network,
        "--netwotk": argparse.sbo_network,
        "-b": argparse.pkg_blacklist,
        "--blacklist": argparse.pkg_blacklist,
        "-q": argparse.pkg_queue,
        "--queue": argparse.pkg_queue,
        "-i": argparse.bin_install,
        "--installpkg": argparse.bin_install,
        "-u": argparse.bin_upgrade,
        "--upgradepkg": argparse.bin_upgrade,
        "-r": argparse.bin_remove,
        "--removepkg": argparse.bin_remove,
        "-f": argparse.bin_find,
        "--find": argparse.bin_find,
        "-F": argparse.pkg_find,
        "--FIND": argparse.pkg_find,
        "-p": argparse.pkg_desc,
        "--desc": argparse.pkg_desc,
        "-d": argparse.pkg_contents,
        "--display": argparse.pkg_contents,
        "-g": argparse.congiguration,
        "--config": argparse.congiguration
    }
    try:
        arguments[args[0]]()
    except KeyError:
        usage("", 1)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# sbo_arch.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.__metadata__ import MetaData as _meta_


class SBoArch:
    """Manage computer architecture for sbo repository
    """
    def __init__(self):
        self.meta = _meta_
        self.arch = self.meta.arch
        # Architectures
        self.x86 = "i586"
        self.arm = "arm"

    def get(self):
        """Return sbo arch
        """
        if self.arch.startswith("i") and self.arch.endswith("86"):
            self.arch = self.x86
        elif self.meta.arch.startswith("arm"):
            self.arch = self.arm
        return self.arch



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# build.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import re
import sys
import time
import shutil
import tarfile
import itertools
import subprocess
import multiprocessing

from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.checksum import check_md5
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.sbo.greps import SBoGrep


class BuildPackage(Utils):
    """Build SBo packages from source
    """
    def __init__(self, script, sources, path, auto):
        self.script = script
        self.sources = sources
        self._check_sources()
        self.path = path
        self.auto = auto
        self.meta = _meta_
        self.msg = Msg()
        self._SOURCES = self.meta.SBo_SOURCES
        self.prgnam = self.script[:-7]
        self.log_file = f"build_{self.prgnam}_log"
        self.sbo_logs = self.meta.log_path + "sbo/"
        self.build_logs = self.sbo_logs + "build_logs/"
        self.start_log_time = time.strftime("%H:%M:%S")
        self.start_time = time.time()
        if not os.path.exists(self.meta.log_path):
            os.mkdir(self.meta.log_path)
        if not os.path.exists(self.sbo_logs):
            os.mkdir(self.sbo_logs)
        if not os.path.exists(self.build_logs):
            os.mkdir(self.build_logs)

    def build(self):
        """
        Build package from source and create log
        file in path /var/log/slpkg/sbo/build_logs/.
        Also check md5sum calculates.
        """
        try:
            self._delete_dir()
            try:
                tar = tarfile.open(self.script)
            except Exception as err:
                raise SystemExit(err)
            tar.extractall()
            tar.close()
            self._makeflags()
            self._delete_sbo_tar_gz()
            self._create_md5_dict()
            if not self.auto:
                os.chdir(self._SOURCES)
            for src in self.sources:
                if not os.path.isfile(src):
                    continue
                # fix build sources with spaces
                src = src.replace("%20", " ")
                check_md5(self.sbo_md5[src], src)
                # copy source and fix passing char '+' from file name
                shutil.copy2(src, self.path + self.prgnam)
            os.chdir(self.path + self.prgnam)
            # change permissions
            subprocess.call(f"chmod +x {self.prgnam}.SlackBuild", shell=True)
            pass_var = self._pass_variable()
            if self.meta.sbo_build_log in ["on", "ON"]:
                if os.path.isfile(self.build_logs + self.log_file):
                    os.remove(self.build_logs + self.log_file)
                # start log write
                log_head(self.build_logs, self.log_file, self.start_log_time)
                subprocess.Popen(f"{' '.join(pass_var)} ./{self.prgnam}.SlackBuild 2>&1 | tee -a "
                                 f"{self.build_logs}{self.log_file}", shell=True, stdout=sys.stdout).communicate()
                sum_time = build_time(self.start_time)
                # write end in log file
                log_end(self.build_logs, self.log_file, sum_time)
                print(f"Total build time for the package {self.prgnam} : {sum_time}\n")
            else:
                subprocess.call(f"{' '.join(pass_var)} ./{self.prgnam}.SlackBuild", shell=True)
            os.chdir(self.path)
        except (KeyboardInterrupt, KeyError) as e:
            print(e)   # (OSError, IOError, KeyError):
            self.msg.pkg_not_found("\n", self.prgnam, "Wrong file", "\n")

    def _check_sources(self):
        """Fix filenames with char +
        """
        new_sources = []
        for src in self.sources:
            new_sources.append(self.fix_file_name(src))
        self.sources = new_sources

    def _create_md5_dict(self):
        """Create md5 dictionary per source
        """
        self.sbo_md5 = {}
        md5_lists = SBoGrep(self.prgnam).checksum()
        for src, md5 in itertools.zip_longest(self.sources, md5_lists):
            self.sbo_md5[src] = md5

    def _makeflags(self):
        """Set variable MAKEFLAGS with the numbers of
        processors
        """
        if self.meta.makeflags in ["on", "ON"]:
            cpus = multiprocessing.cpu_count()
            os.environ["MAKEFLAGS"] = f"-j{cpus}"

    def _pass_variable(self):
        """Return enviroment variables
        """
        pass_var = []
        for var in os.environ.keys():
            expVAR = var.split("_")
            if expVAR[0] == self.prgnam.upper() and expVAR[1] != "PATH":
                pass_var.append(f"{expVAR[1]}={os.environ[var]}")
        return pass_var

    def _delete_sbo_tar_gz(self):
        """Delete slackbuild tar.gz file after untar
        """
        if not self.auto and os.path.isfile(self.meta.build_path + self.script):
            os.remove(self.meta.build_path + self.script)

    def _delete_dir(self):
        """Delete old folder if exists before start build
        """
        if not self.auto and os.path.isdir(self.meta.build_path + self.prgnam):
            shutil.rmtree(self.meta.build_path + self.prgnam)


def log_head(path, log_file, log_time):
    """
    write headers to log file
    """
    with open(path + log_file, "w") as log:
        log.write("#" * 79 + "\n\n")
        log.write("File : " + log_file + "\n")
        log.write("Path : " + path + "\n")
        log.write("Date : " + time.strftime("%d/%m/%Y") + "\n")
        log.write("Time : " + log_time + "\n\n")
        log.write("#" * 79 + "\n\n")


def log_end(path, log_file, sum_time):
    """
    append END tag to a log file
    """
    with open(path + log_file, "a") as log:
        log.seek(2)
        log.write("#" * 79 + "\n\n")
        log.write("Time : " + time.strftime("%H:%M:%S") + "\n")
        log.write(f"Total build time : {sum_time}\n")
        log.write(" " * 38 + "E N D\n\n")
        log.write("#" * 79 + "\n\n")


def build_time(start_time):
    """
    Calculate build time per package
    """
    diff_time = round(time.time() - start_time, 2)
    if diff_time <= 59.99:
        sum_time = str(diff_time) + " Sec"
    elif diff_time > 59.99 and diff_time <= 3599.99:
        sum_time = round(diff_time / 60, 2)
        sum_time_list = re.findall(r"\d+", str(sum_time))
        sum_time = (f"{sum_time_list[0]} Min {sum_time_list[1]} Sec")
    elif diff_time > 3599.99:
        sum_time = round(diff_time / 3600, 2)
        sum_time_list = re.findall(r"\d+", str(sum_time))
        sum_time = (f"{sum_time_list[0]} Hours {sum_time_list[1]} Min")
    return sum_time



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# messages.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import itertools

from slpkg.__metadata__ import MetaData as _meta_


class Msg:
    """Messages control
    """
    def __init__(self):
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.grey = _meta_.color["GREY"]
        self.red = _meta_.color["RED"]
        self.cyan = _meta_.color["CYAN"]
        self.endc = _meta_.color["ENDC"]

    def pkg_not_found(self, bol, pkg, message, eol):
        """Print message when package not found
        """
        print(f"{bol}No such package {pkg}: {message}{eol}")

    def pkg_found(self, prgnam):
        """Print message when package found
        """
        print(f"| Package {prgnam} is already installed")

    def pkg_installed(self, pkg):
        """Print message when package installed
        """
        print(f"| Package {pkg} installed")

    def build_FAILED(self, prgnam):
        """Print error message if build failed
        """
        self.template(78)
        print(f"| Some error on the package {prgnam} "
              f"[ {self.red}FAILED{self.endc} ]")
        self.template(78)
        print(f"| See the log file in '{self.cyan}"
              f"/var/log/slpkg/sbo/build_logs{self.endc}' "
              f"directory or read the README file")
        self.template(78)
        print()   # new line at end

    def template(self, max_len):
        """Print template
        """
        print("+" + "=" * max_len)

    def checking(self):
        """Message checking
        """
        print(f"{self.grey}Checking...{self.endc}  ", end="", flush=True)

    def reading(self):
        """Message reading
        """
        print(f"{self.grey}Reading package lists...{self.endc}  ",
              end="", flush=True)

    def resolving(self):
        """Message resolving
        """
        print(f"{self.grey}Resolving dependencies...{self.endc}  ",
              end="", flush=True)

    def done(self):
        """Message done
        """
        print(f"\b{self.green}Done{self.endc}\n", end="")

    def pkg(self, count):
        """Print singular plural
        """
        message = "package"
        if count > 1:
            message = message + "s"
        return message

    def not_found(self, if_upgrade):
        """Message not found packages
        """
        if if_upgrade:
            print("\nNot found packages for upgrade\n")
        else:
            print("\nNot found packages for installation\n")

    def upg_inst(self, if_upgrade):
        """Message installing or upgrading
        """
        if not if_upgrade:
            print("Installing:")
        else:
            print("Upgrading:")

    def answer(self):
        """Message answer
        """
        if self.meta.default_answer in ["y", "Y"]:
            answer = self.meta.default_answer
        else:
            try:
                answer = input("Would you like to continue [y/N]? ")
            except EOFError:
                raise SystemExit("\n")
        return answer

    def security_pkg(self, pkg):
        """Warning message for some special reasons
        """
        print()
        self.template(78)
        print(f"| {' ' * 27}{self.red}*** WARNING ***{self.endc}")
        self.template(78)
        print(f"| Before proceed with the package '{pkg}' will you must read\n"
              f"| the README file. You can use the command "
              f"'slpkg -n {pkg}'")
        self.template(78)
        print()

    def reference(self, install, upgrade):
        """Reference list with packages installed
        and upgraded
        """
        self.template(78)
        print(f"| Total {len(install)} {self.pkg(len(install))} installed and "
              f"{len(upgrade)} {self.pkg(len(upgrade))} upgraded")
        self.template(78)
        for installed, upgraded in itertools.zip_longest(install, upgrade):
            if upgraded:
                print(f"| Package {upgraded} upgraded successfully")
            if installed:
                print(f"| Package {installed} installed successfully")
        self.template(78)
        print()

    def matching(self, packages):
        """Message for matching packages
        """
        print(f"\nNot found package with the name "
              f"[ {self.cyan}{''.join(packages)}{self.endc} ]. "
              "Matching packages:\nNOTE: Not dependencies are resolved\n")



import unittest
from slpkg.sbo.greps import SBoGrep


class TestSBoGreps(unittest.TestCase):

    def setUp(self):
        self.grep = SBoGrep('Flask')

    def test_source(self):
        """Test package source
        """
        source = self.grep.source()
        flask_source = ('https://files.pythonhosted.org/packages/4e/0b/'
                        'cb02268c90e67545a0e3a37ea1ca3d45de3aca43ceb7dbf'
                        '1712fb5127d5d/Flask-1.1.2.tar.gz')
        self.assertEqual(source, flask_source)

    def test_requires(self):
        """Test package requires
        """
        requires = self.grep.requires()
        flask_dep = ['werkzeug', 'python3-itsdangerous', 'click']
        self.assertListEqual(requires, flask_dep)

    def test_version(self):
        """Test package version
        """
        version = self.grep.version()
        flask_ver = '1.1.2'
        self.assertEqual(version, flask_ver)

    def test_checksum(self):
        """Test package checksum
        """
        checksum = self.grep.checksum()
        flask_md5 = ['0da4145d172993cd28a6c619630cc19c']
        self.assertListEqual(checksum, flask_md5)

    def test_description(self):
        """Test package description
        """
        desc = self.grep.description()
        flask_desc = 'Flask (Microframework for Python)'
        self.assertEqual(desc, flask_desc)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# repolist.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.messages import Msg
from slpkg.repositories import Repo
from slpkg.__metadata__ import MetaData as _meta_


class RepoList:
    """List of repositories
    """
    def __init__(self):
        self.meta = _meta_
        self.green = self.meta.color["GREEN"]
        self.red = self.meta.color["RED"]
        self.grey = self.meta.color["GREY"]
        self.endc = self.meta.color["ENDC"]
        self.msg = Msg()
        self.all_repos = Repo().default_repository()
        self.all_repos["slack"] = Repo().slack()
        self.all_repos.update(Repo().custom_repository())

    def repos(self):
        """View or enabled or disabled repositories
        """
        def_cnt, cus_cnt = 0, 0
        self.msg.template(78)
        print("{0}{1}{2}{3}{4}{5}{6}".format(
            "| Repo_id", " " * 2,
            "Repo_URL", " " * 44,
            "Default", " " * 3,
            "Status"))
        self.msg.template(78)
        for repo_id, repo_URL in sorted(self.all_repos.items()):
            status, COLOR = "disabled", self.red
            default = "yes"
            if len(repo_URL) > 49:
                repo_URL = repo_URL[:48] + "~"
            if repo_id in self.meta.repositories:
                def_cnt += 1
                status, COLOR = "enabled", self.green
            if repo_id not in self.meta.default_repositories:
                cus_cnt += 1
                default = "no"
            print("  {0}{1}{2}{3}{4}{5}{6}{7:>8}{8}".format(
                repo_id, " " * (9 - len(repo_id)),
                repo_URL, " " * (52 - len(repo_URL)),
                default, " " * (8 - len(default)),
                COLOR, status, self.endc))

        print("\nRepositories summary")
        print("=" * 79)
        print(f"{self.grey}{def_cnt}/{len(self.all_repos)} enabled default "
              f"repositories and {cus_cnt} custom.")
        print("Edit the file '/etc/slpkg/repositories.conf' for enable "
              "and disable default\nrepositories or run 'slpkg "
              f"repo-enable' command.{self.endc}")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# md5sum.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import hashlib


def md5(source):
    """Return MD5 Checksum
    """
    # fix passing char '+' from source
    source = source.replace("%2B", "+")
    with open(source, "rb") as file_to_check:
        data = file_to_check.read()
        return hashlib.md5(data).hexdigest()


#!/usr/bin/python3
# -*- coding: utf-8 -*-

# queue.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from collections import OrderedDict

from slpkg.utils import Utils
from slpkg.downloader import Download
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package
from slpkg.pkg.build import BuildPackage
from slpkg.pkg.manager import PackageManager

from slpkg.sbo.greps import SBoGrep
from slpkg.sbo.compressed import SBoLink
from slpkg.sbo.search import sbo_search_pkg
from slpkg.sbo.slack_find import slack_package


class QueuePkgs(Utils):
    """Manage SBo packages, add or remove for building or
    installation
    """
    def __init__(self):
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.endc = _meta_.color["ENDC"]
        self.queue = self.meta.lib_path + "queue/"
        self.queue_list = self.queue + "queue_list"
        self._SOURCES = self.meta.SBo_SOURCES
        self.touch()

    def touch(self):
        """Creating the directories and the queue file
        """
        queue_file = [
            "# In this file you can create a list of\n",
            "# packages you want to build or install.\n",
            "#\n"]
        if not os.path.exists(self.meta.lib_path):
            os.mkdir(self.meta.lib_path)
        if not os.path.exists(self.queue):
            os.mkdir(self.queue)
        if not os.path.isfile(self.queue_list):
            with open(self.queue_list, "w") as queue:
                for line in queue_file:
                    queue.write(line)
        self.queued = self.read_file(self.queue_list)

    def packages(self):
        """Return queue list from /var/lib/queue/queue_list
        file.
        """
        for read in self.queued.splitlines():
            read = read.lstrip()
            if not read.startswith("#"):
                yield read.replace("\n", "")

    def listed(self):
        """Print packages from queue
        """
        print("Packages in the queue:")
        for pkg in self.packages():
            print(f"{self.green}{pkg}{self.endc}")
        print()

    def add(self, pkgs):
        """Add packages in queue if not exist
        """
        queue_list = list(self.packages())
        pkgs = list(OrderedDict.fromkeys(pkgs))
        print("Add packages in the queue:")
        with open(self.queue_list, "a") as queue:
            for pkg in pkgs:
                find = sbo_search_pkg(pkg)
                if pkg not in queue_list and find is not None:
                    print(f"{self.green}{pkg}{self.endc}")
                    queue.write(pkg + "\n")
                else:
                    print(f"{self.red}{pkg}{self.endc}")
        print()

    def remove(self, pkgs):
        """Remove packages from queue
        """
        print("Remove packages from the queue:")
        with open(self.queue_list, "w") as queue:
            for line in self.queued.splitlines():
                if line not in pkgs:
                    queue.write(line + "\n")
                else:
                    print(f"{self.red}{line}{self.endc}")
        print()

    def build(self):
        """Build packages from queue
        """
        sources = []
        packages = list(self.packages())
        if packages:
            for pkg in packages:
                if pkg not in SBoGrep(pkg).names():
                    raise SystemExit(f"\nPackage '{pkg}' was not found in "
                                     f"the SBo repository\n")
                if not os.path.exists(self.meta.build_path):
                    os.mkdir(self.meta.build_path)
                if not os.path.exists(self._SOURCES):
                    os.mkdir(self._SOURCES)
                sbo_url = sbo_search_pkg(pkg)
                sbo_dwn = SBoLink(sbo_url).tar_gz()
                source_dwn = SBoGrep(pkg).source().split()
                os.chdir(self.meta.build_path)
                script = sbo_dwn.split("/")[-1]
                Download(self.meta.build_path, sbo_dwn.split(),
                         repo="sbo").start()
                for src in source_dwn:
                    Download(self._SOURCES, src.split(), repo="sbo").start()
                    sources.append(src.split("/")[-1])
                BuildPackage(script, sources, self.meta.build_path,
                             auto=False).build()
        else:
            raise SystemExit("\nPackages not found in the queue for "
                             "building\n")

    def install(self):
        """Install packages from queue
        """
        packages = list(self.packages())
        if packages:
            print()   # new line at start
            for pkg in packages:
                ver = SBoGrep(pkg).version()
                prgnam = f"{pkg}-{ver}"
                if find_package(prgnam, self.meta.output):
                    binary = slack_package(prgnam)
                    PackageManager(binary).upgrade(flag="--install-new")
                else:
                    print(f"\nPackage {prgnam} not found in the "
                          f"{self.meta.output} for installation\n")
        else:
            raise SystemExit("\nPackages not found in the queue for "
                             "installation\n")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# arguments.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.repolist import RepoList
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.slack.slack_version import slack_ver


def header():
    """help header message"""
    print(f"slpkg - version {_meta_.__version__} | Slackware release: "
          f"{_meta_.slack_rel} - {slack_ver()}\n")


def options():
    """Slpkg is a user-friendly package manager for Slackware installations

Usage: slpkg [COMMANDS|OPTIONS] {repository|package...}

                                                   _       _
                                               ___| |_ __ | | ____ _
                                              / __| | '_ \| |/ / _` |
                                              \__ \ | |_) |   < (_| |
                                              |___/_| .__/|_|\_\__, |
                                                    |_|        |___/
                                             _Slackware package manager_______

Commands:
   update, --repos=[...]                     Run this command to update all
                                             the packages lists.

   upgrade, --repos=[...]                    Delete and recreate all packages
                                             lists.

   repo-add [repository name] [URL]          Add custom repository.

   repo-remove [repository]                  Remove custom repository.

   repo-enable                               Enable or disable default
                                             repositories via dialog utility.

   repo-list                                 Print a list of all the
                                             repositories.

   repo-info [repository]                    Get information about a
                                             repository.

   update slpkg                              Upgrade the program directly from
                                             repository.

   health, --silent                          Health check installed packages.

   deps-status, --tree, --graph=[type]       Print dependencies status used by
                                             packages or drawing dependencies
                                             diagram.

   new-config                                Manage .new configuration files.

   clean-tmp                                 Clean the tmp/ directory from
                                             downloaded packages and sources.
Optional arguments:
  -h | --help                                Print this help message and exit.

  -v | --version                             Print program version and exit.

  -a | --autobuild, [script] [source...]     Auto build SBo packages.
                                             If you have already downloaded the
                                             script and the source code you can
                                             build a new package with this
                                             command.

  -b | --blacklist, [package...] --add,      Manage packages in the blacklist.
       --remove, list                        Add or remove packages and print
                                             the list. Each package is added
                                             here will not be accessible by the
                                             program.

  -q | --queue, [package...] --add,          Manage SBo packages in the queue.
       --remove, list, build, install,       Add or remove and print the list
       build-install                         of packages. Build and then
                                             install the packages from the
                                             queue.

  -g | --config, print, edit, reset          Configuration file management.
                                             Print, edit the configuration file
                                             or reset in the default values.

  -l | --list, [repository], --index,        Print a list of all available
       --installed, --name                   packages from repository, index or
                                             print only packages installed on
                                             the system.

  -c | --check, [repository], --upgrade,     Check for updated packages from
       --rebuild --skip=[...],               the repositories and upgrade or
       --resolve-off, --checklist            install with all dependencies.

  -s | --sync, [repository] [package...],    Sync packages. Install packages
       --rebuild, --reinstall,               directly from remote repositories
       --resolve-off, --download-only,       with all dependencies.
       --directory-prefix=[dir],
       --case-ins, --patches

  -t | --tracking, [repository] [package],   Tracking package dependencies and
       --check-deps, --graph=[type],         print package dependencies tree
       --case-ins                            with highlight if packages is
                                             installed. Also check if
                                             dependencies used or drawing
                                             dependencies diagram.

  -p | --desc, [repository] [package],       Print description of a package
       --color=[]                            directly from the repository and
                                             change color text.

  -n | --network, [package], --checklist,    View a standard of SBo page in
       --case-ins                            terminal and manage multiple
                                             options like reading, downloading,
                                             building, installation, etc.

  -F | --FIND, [package...], --case-ins      Find packages from each enabled
                                             repository and view results.

  -f | --find, [package...], --case-ins,     Find and print installed packages
       --third-party                         reporting the size and the sum.

  -i | --installpkg, [options] [package...]  Installs single or multiple *.tgz
       options=[--warn, --md5sum, --root,    (or .tbz, .tlz, .txz) Slackware
       --infobox, --menu, --terse, --ask,    binary packages designed for use
       --priority, --tagfile]                with the Slackware Linux
                                             distribution onto your system.

  -u | --upgradepkg, [options] [package...]  Upgrade single or multiple
       options=[--dry-run, --install-new,    Slackware binary packages from
       --reinstall, --verbose]               an older version to a newer one.

  -r | --removepkg, [options] [package...],  Removes a previously installed
       --deps, --check-deps, --tag,          Slackware binary packages,
       --checklist, --third-party            while writing a progress report
       options=[-warn, -preserve, -copy,     to the standard output.
       -keep]                                Use only package name.

  -d | --display, [package...]               Display the contents of installed
                                             packages and file list.

You can read more about slpkg from manpage or see examples from readme file.
Issues: https://gitlab.com/dslackw/slpkg/issues
Homepage: https://dslackw.gitlab.io/slpkg/
"""
    header()
    print(options.__doc__)


def usage(repo, stderr):
    """Usage: slpkg [COMMANDS|OPTIONS] {repository|package...}

             Commands:
             [update, --repos=[...]]
             [upgrade, --repos=[...]]
             [repo-add [repository name] [URL]]
             [repo-remove [repository]]
             [repo-enable]
             [repo-list]
             [repo-info [repository]]
             [update [slpkg]]
             [health, --silent]
             [deps-status, --tree, --graph=[type]]
             [new-config]
             [clean-tmp]

             Optional arguments:
             [-h] [-v]
             [-a [script] [sources...]]
             [-b [package...] --add, --remove,
                 [list]]
             [-q [package...] --add, --remove,
                 [list, build, install, build-install]]
             [-g [print, edit, reset]]
             [-l [repository], --index, --installed, --name]
             [-c [repository], --upgrade, --rebuild, --skip=[...],
                               --resolve-off, --checklist]
             [-s [repository] [package...], --rebuild, --reinstall,
                                            --resolve-off, --download-only,
                                            --directory-prefix=[dir],
                                            --case-ins, --patches]
             [-t [repository] [package], --check-deps, --graph=[type],
                                         --case-ins]
             [-p [repository] [package], --color=[]]
             [-n [SBo package], --checklist, --case-ins]
             [-F [package...], --case-ins]
             [-f [package...], --case-ins, --third-party]
             [-i [options] [package...]]
             [-u [options] [package...]]
             [-r [options] [package...], --deps, --check-deps, --tag,
                                         --checklist, --third-party]
             [-d [package...]]
             """
    if repo and repo not in _meta_.repositories:
        error_repo = ""
        all_repos = RepoList().all_repos.keys()
        del RepoList().all_repos
        if repo in all_repos:
            error_repo = (f"slpkg: Error: Repository '{repo}' is not activated"
                          "\n")
        else:
            error_repo = (f"slpkg: Error: Repository '{repo}' does not exist"
                          "\n")
        raise SystemExit(f"\n{error_repo}")
    print(usage.__doc__)
    print("For more information try 'slpkg -h, --help' or view manpage\n")
    if stderr == 1:
        raise SystemExit(stderr)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# repoinfo.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.sizes import units
from slpkg.utils import Utils
from slpkg.repositories import Repo
from slpkg.repolist import RepoList
from slpkg.__metadata__ import MetaData as _meta_


class RepoInfo(Utils):
    """Repository information
    """
    def __init__(self):
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.endc = _meta_.color["ENDC"]
        self.repo = Repo()
        self.form = {
            "Last updated:": "",
            "Number of packages:": "",
            "Repo id:": "",
            "Default:": "",
            "Repo url:": "",
            "Status:": "",
            "Total compressed packages:": "",
            "Total uncompressed packages:": ""
        }
        self.meta = _meta_
        self.all_repos = self.repo.default_repository()
        self.all_repos["slack"] = self.repo.slack()
        self.all_repos.update(self.repo.custom_repository())
        del RepoList().all_repos

    def view(self, repo):
        """
        View repository information
        """
        status = f"{self.red}disabled{self.endc}"
        self.form["Status:"] = status
        self.form["Default:"] = "no"

        if repo in self.meta.default_repositories:
            self.form["Default:"] = "yes"

        if (repo in self.meta.repositories and
            os.path.isfile(f"{self.meta.lib_path}"
                           f"{repo}_repo/PACKAGES.TXT")):
            status = f"{self.green}enabled{self.endc}"

            if repo != "sbo":
                data = self.repository_data(repo)
                size = units(data[1], data[2])
                self.form["Repo id:"] = repo
                self.form["Repo url:"] = self.all_repos[repo]
                self.form["Total compressed packages:"] = f"{str(size[1][0])} {str(size[0][0])}"
                self.form["Total uncompressed packages:"] = f"{str(size[1][1])} {str(size[0][1])}"
                self.form["Number of packages:"] = data[0]
                self.form["Status:"] = status
                self.form["Last updated:"] = data[3]

        elif (repo == "sbo" and os.path.isfile(
                f"{self.meta.lib_path}{repo}_repo/SLACKBUILDS.TXT")):
            status = f"{self.green}enabled{self.endc}"
            sum_sbo_pkgs = 0

            for line in (self.read_file(
                    f"{self.meta.lib_path}sbo_repo/SLACKBUILDS."
                    "TXT").splitlines()):
                if line.startswith("SLACKBUILD NAME: "):
                    sum_sbo_pkgs += 1

            changelog_txt = self.read_file(
                f"{self.meta.log_path}sbo/ChangeLog.txt")
            last_upd = changelog_txt.split("\n", 1)[0]

            self.form["Repo id:"] = repo
            self.form["Repo url:"] = self.all_repos[repo]
            self.form["Total compressed packages:"] = ""
            self.form["Total uncompressed packages:"] = ""
            self.form["Number of packages:"] = sum_sbo_pkgs
            self.form["Status:"] = status
            self.form["Last updated:"] = last_upd

        for key, value in sorted(self.form.items()):
            print(f"{self.green}{key}{self.endc} {value}")

    def repository_data(self, repo):
        """Grap data packages
        """
        sum_pkgs, size, unsize, last_upd = 0, [], [], ""
        f = f"{self.meta.lib_path}{repo}_repo/PACKAGES.TXT"

        for line in self.read_file(f).splitlines():
            if line.startswith("PACKAGES.TXT;"):
                last_upd = line[14:].strip()
            if line.startswith("PACKAGE NAME:"):
                sum_pkgs += 1
            if line.startswith("PACKAGE SIZE (compressed):  "):
                size.append(line[28:-2].strip())
            if line.startswith("PACKAGE SIZE (uncompressed):  "):
                unsize.append(line[30:-2].strip())

        if repo in ["salix", "slackl"]:
            log = self.read_file(
                f"{self.meta.log_path}{repo}/ChangeLog.txt")
            last_upd = log.split("\n", 1)[0]

        return [sum_pkgs, size, unsize, last_upd]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# greps.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from functools import lru_cache
from slpkg.utils import Utils
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.slack.slack_version import slack_ver


def repo_data(PACKAGES_TXT, repo, flag):
    """Grap data packages
    """
    (name, location, size, unsize,
     rname, rlocation, rsize, runsize) = ([] for i in range(8))
    for line in PACKAGES_TXT.splitlines():
        if line.startswith("PACKAGE NAME:"):
            name.append(line[15:].strip())
        if line.startswith("PACKAGE LOCATION:"):
            location.append(line[21:].strip())
        if line.startswith("PACKAGE SIZE (compressed):"):
            size.append(line[28:-2].strip())
        if line.startswith("PACKAGE SIZE (uncompressed):"):
            unsize.append(line[30:-2].strip())

    if repo == "slack" and "--upgrade" not in flag:
        (rname,
         rlocation,
         rsize,
         runsize
         ) = slack_filter(name, location, size, unsize, flag)
    elif repo == "rlw":
        (rname,
         rlocation,
         rsize,
         runsize
         ) = rlw_filter(name, location, size, unsize)
    elif repo == "alien":
        (rname,
         rlocation,
         rsize,
         runsize
         ) = alien_filter(name, location, size, unsize)
    elif repo == "rested":
        (rname,
         rlocation,
         rsize,
         runsize
         ) = rested_filter(name, location, size, unsize)
    elif repo == "ktown":
        (rname,
         rlocation,
         rsize,
         runsize
         ) = ktown_filter(name, location, size, unsize)
    else:
        rname, rlocation, rsize, runsize = name, location, size, unsize
    return [rname, rlocation, rsize, runsize]


def slack_filter(name, location, size, unsize, flag):
    """Slackware filter seperate packages from patches/ directory
    """
    (fname, flocation, fsize, funsize) = ([] for i in range(4))

    if "--patches" not in flag:
        for n, l, s, u in zip(name, location, size, unsize):
            if f"_slack{slack_ver()}" not in n:
                fname.append(n)
                flocation.append(l)
                fsize.append(s)
                funsize.append(u)

    if "--patches" in flag:
        for n, l, s, u in zip(name, location, size, unsize):
            if f"_slack{slack_ver()}" in n:
                fname.append(n)
                flocation.append(l)
                fsize.append(s)
                funsize.append(u)

    return [fname, flocation, fsize, funsize]


def rlw_filter(name, location, size, unsize):
    """Filter rlw repository data
    """
    arch = _meta_.arch
    if arch.startswith("i") and arch.endswith("86"):
        arch = "i486"
    (fname, flocation, fsize, funsize) = ([] for i in range(4))
    for n, l, s, u in zip(name, location, size, unsize):
        loc = l.split("/")
        if arch == loc[-1]:
            fname.append(n)
            flocation.append(l)
            fsize.append(s)
            funsize.append(u)
    return [fname, flocation, fsize, funsize]


def alien_filter(name, location, size, unsize):
    """Fix to avoid packages include in slackbuilds folder
    """
    (fname, flocation, fsize, funsize) = ([] for i in range(4))
    for n, l, s, u in zip(name, location, size, unsize):
        if "slackbuilds" != l:
            fname.append(n)
            flocation.append(l)
            fsize.append(s)
            funsize.append(u)
    return [fname, flocation, fsize, funsize]


def rested_filter(name, location, size, unsize):
    """Filter Alien"s repository data
    """
    ver = slack_ver()
    if _meta_.slack_rel == "current":
        ver = "current"
    path_pkg = "pkg"
    if _meta_.arch == "x86_64":
        path_pkg = "pkg64"
    (fname, flocation, fsize, funsize) = ([] for i in range(4))
    for n, l, s, u in zip(name, location, size, unsize):
        if path_pkg == l.split("/")[-2] and ver == l.split("/")[-1]:
            fname.append(n)
            flocation.append(l)
            fsize.append(s)
            funsize.append(u)
    return [fname, flocation, fsize, funsize]


def ktown_filter(name, location, size, unsize):
    """Filter Alien"s ktown repository data
    """
    ver = slack_ver()
    if _meta_.slack_rel == "current":
        ver = "current"
    path_pkg = "x86"
    if _meta_.arch == "x86_64":
        path_pkg = _meta_.arch
    (fname, flocation, fsize, funsize) = ([] for i in range(4))
    for n, l, s, u in zip(name, location, size, unsize):
        if (path_pkg in l and _meta_.ktown_kde_repo[1:-1] in l and
                l.startswith(ver)):
            fname.append(n)
            flocation.append(l)
            fsize.append(s)
            funsize.append(u)
    return [fname, flocation, fsize, funsize]


class Requires:

    def __init__(self, name, repo):
        self.name = name
        self.repo = repo

    def get_deps(self):
        """Grap package requirements from repositories
        """
        if self.repo == "rlw":
            dependencies = {}
            rlw_deps = Utils().read_file(_meta_.conf_path + "rlworkman.deps")
            for line in rlw_deps.splitlines():
                if line and not line.startswith("#"):
                    pkgs = line.split(":")
                    dependencies[pkgs[0]] = pkgs[1]
            if self.name in dependencies.keys():
                return dependencies[self.name].split()
            else:
                return ""
        else:
            PACKAGES_TXT = Utils().read_file(f"{_meta_.lib_path}"
                                             f"{self.repo}_repo/PACKAGES.TXT")
            for line in PACKAGES_TXT.splitlines():
                if line.startswith("PACKAGE NAME:"):
                    pkg_name = split_package(line[14:].strip())[0]
                if line.startswith("PACKAGE REQUIRED:"):
                    if pkg_name == self.name:
                        if line[18:].strip():
                            return self._req_fix(line)

    @lru_cache
    def _req_fix(self, line):
        """Fix slacky and salix requirements because many dependencies
        splitting with "," and others with "|"
        """
        deps = []
        for dep in line[18:].strip().split(","):
            dep = dep.split("|")
            if self.repo == "slacky":
                if len(dep) > 1:
                    for d in dep:
                        deps.append(d.split()[0])
                dep = "".join(dep)
                deps.append(dep.split()[0])
            else:
                if len(dep) > 1:
                    for d in dep:
                        deps.append(d)
                deps.append(dep[0])
        return deps



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# graph.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import subprocess


# class ImportErrorGraphEasy(Exception):
#    def __init__(self, GraphEasyImportError):
#        Exception.__init__(self, "graph-easy required")
#        self.GraphEasyImportError = GraphEasyImportError


class Graph:
    """Drawing dependencies diagram
    """
    def __init__(self, image):
        self.image = image
        self.file_format = [
            ".bmp", ".canon", ".cmap", ".cmapx", ".cmapx_np", ".dot",
            ".eps", ".fig", ".gd", ".gd2", ".gif", ".gtk", ".gv", ".ico",
            ".imap", ".imap_np", ".ismap", ".jpe", ".jpeg", ".jpg", ".pdf",
            ".pic", ".plain", ".plain-ext", ".png", ".pov", ".ps", ".ps2",
            ".svg", ".svgz", ".tif", ".tiff", ".tk", ".vml", ".vmlz",
            ".vrml", ".wbmp", ".x11", ".xdot", ".xlib"
        ]

    def dependencies(self, deps_dict):
        """Generate graph file with depenndencies map tree
        """
        try:
            import pygraphviz as pgv
        except ImportError:
            if (self.image == "ascii"
                    and not os.path.isfile("/usr/bin/graph-easy")):
                print("Require 'grap_easy': "
                      "Install with 'slpkg -s sbo graph-easy'")
            else:
                raise SystemExit("Require 'pygraphviz: "
                                 "Install with 'slpkg -s sbo pygraphviz'")
        if self.image != "ascii":
            self.check_file()
        try:
            G = pgv.AGraph(deps_dict)
            G.layout(prog="fdp")
            if self.image == "ascii":
                G.write(f"{self.image}.dot")
                self.graph_easy()
            G.draw(self.image)
        except IOError:
            raise SystemExit(1)
        if os.path.isfile(self.image):
            print(f"Graph image file '{self.image}' created")
        raise SystemExit()

    def check_file(self):
        """Check for file format and type
        """
        try:
            image_type = f".{self.image.split('.')[1]}"
            if image_type not in self.file_format:
                raise SystemExit(f"Format: '{self.image.split('.')[1]}' not "
                                 f"recognized. Use one of them:\n"
                                 f"{', '.join(self.file_format)}")
        except IndexError:
            raise SystemExit("slpkg: Error: Image file suffix missing")

    def graph_easy(self):
        """Draw ascii diagram. graph-easy perl module require
        """
        if not os.path.isfile("/usr/bin/graph-easy"):
            print("Require 'graph-easy': Install with 'slpkg -s sbo"
                  " graph-easy'")
            self.remove_dot()
            raise SystemExit(1)
        subprocess.call(f"graph-easy {self.image}.dot", shell=True)
        self.remove_dot()
        raise SystemExit(1)

    def remove_dot(self):
        """Remove .dot files
        """
        if os.path.isfile(f"{self.image}.dot"):
            os.remove(f"{self.image}.dot")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# search.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.repositories import Repo
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.slack.slack_version import slack_ver


def sbo_search_pkg(name):
    """Search for package path in SLACKBUILDS.TXT file and
    return url
    """
    url = ""
    utils = Utils()
    repo = Repo()
    sbo = repo.default_repository()["sbo"]
    sbo_url = f"{sbo}{slack_ver()}/"
    SLACKBUILDS_TXT = utils.read_file(
        f"{_meta_.lib_path}sbo_repo/SLACKBUILDS.TXT")
    for line in SLACKBUILDS_TXT.splitlines():
        if line.startswith("SLACKBUILD LOCATION"):
            sbo_name = (line[23:].split("/")[-1].replace("\n", "")).strip()
            if name == sbo_name:
                url = f"{sbo_url}{line[23:].strip()}/"
    return url



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# read.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.url_read import URL


class ReadSBo:
    """Read SBo files from urls
    """
    def __init__(self, sbo_url):
        self.sbo_url = sbo_url

    def readme(self, sbo_readme):
        """Read SlackBuild README file
        """
        return URL(self.sbo_url + sbo_readme).get_request()

    def info(self, name, sbo_file):
        """Read info file
        """
        return URL(self.sbo_url + name + sbo_file).get_request()

    def slackbuild(self, name, sbo_file):
        """Read SlackBuild file
        """
        return URL(self.sbo_url + name + sbo_file).get_request()

    def doinst(self, doinst_sh):
        """Read SlackBuild doinst.sh
        """
        return URL(self.sbo_url + doinst_sh).get_request()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# init.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import shutil

from slpkg.utils import Utils
from slpkg.repositories import Repo
from slpkg.file_size import FileSize
from slpkg.downloader import Download
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.slack.mirrors import mirrors
from slpkg.slack.slack_version import slack_ver


class Initialization(Utils):
    """Slpkg initialization start all from here. Create local
    package lists and update or upgrade these.
    """
    def __init__(self, check):
        self.check = check
        self.meta = _meta_
        self.arch = _meta_.arch
        self.slack_ver = slack_ver()
        self.def_repos_dict = Repo().default_repository()
        self.conf_path = self.meta.conf_path
        self.log_path = self.meta.log_path
        self.lib_path = self.meta.lib_path
        self.tmp_path = self.meta.tmp_path
        self.build_path = self.meta.build_path
        self._SOURCES = self.meta.SBo_SOURCES
        self.slpkg_tmp_packages = self.meta.slpkg_tmp_packages
        self.slpkg_tmp_patches = self.meta.slpkg_tmp_patches
        self.constructing()

    def constructing(self):
        """Creating the all necessary directories
        """
        paths_basic = [
            self.conf_path,
            self.log_path,
            self.lib_path,
            self.tmp_path
        ]

        paths_extra = [
            self.build_path,
            self._SOURCES,
            self.slpkg_tmp_packages,
            self.slpkg_tmp_patches
        ]

        self.make_dir(paths_basic)
        self.make_dirs(paths_extra)

    def make_dir(self, path: list):
        for p in path:
            if not os.path.exists(p):
                os.mkdir(p)

    def make_dirs(self, path: list):
        for p in path:
            if not os.path.exists(p):
                os.makedirs(p)

    def custom(self, name):
        """Creating user select repository local library
        """
        repo = Repo().custom_repository()[name]
        log = self.log_path + name + "/"
        lib = self.lib_path + f"{name}_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        PACKAGES_TXT = f"{repo}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def slack(self):
        """Creating slack local libraries
        """
        log = self.log_path + "slack/"
        lib = self.lib_path + "slack_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        dirs = ["core/", "extra/", "patches/"]
        self.make_dir([log, lib])
        self.make_dir([f"{lib}{dirs[0]}",
                       f"{lib}{dirs[1]}",
                       f"{lib}{dirs[2]}"])
        PACKAGES_TXT = mirrors(lib_file, "")
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = mirrors(md5_file, "")
        self.EXTRA = mirrors(lib_file, dirs[1])
        self.EXT_CHECKSUMS = mirrors(md5_file, dirs[1])
        self.PATCHES = mirrors(lib_file, dirs[2])
        self.PAT_CHECKSUMS = mirrors(md5_file, dirs[2])
        ChangeLog_txt = mirrors(log_file, "")
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib + dirs[0], PACKAGES_TXT, repo_name)
        self.down(lib + dirs[0], CHECKSUMS_MD5, repo_name)
        self.down(lib + dirs[1], self.EXTRA, repo_name)
        self.down(lib + dirs[1], self.EXT_CHECKSUMS, repo_name)
        self.down(lib + dirs[2], self.PATCHES, repo_name)
        self.down(lib + dirs[2], self.PAT_CHECKSUMS, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)
        self.merge(lib, "PACKAGES.TXT", ["core/PACKAGES.TXT",
                                         "extra/PACKAGES.TXT",
                                         "patches/PACKAGES.TXT"])
        self.merge(lib, "CHECKSUMS.md5", ["core/CHECKSUMS.md5",
                                          "extra/CHECKSUMS.md5",
                                          "patches/CHECKSUMS_md5"])

    def sbo(self):
        """Creating sbo local library
        """
        repo = self.def_repos_dict["sbo"]
        log = self.log_path + "sbo/"
        lib = self.lib_path + "sbo_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "SLACKBUILDS.TXT"
        # lst_file = ""
        # md5_file = ""
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        SLACKBUILDS_TXT = f"{repo}{self.slack_ver}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = ""
        ChangeLog_txt = f"{repo}{self.slack_ver}/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, SLACKBUILDS_TXT, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, SLACKBUILDS_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def rlw(self):
        """Creating rlw local library
        """
        repo = self.def_repos_dict["rlw"]
        log = self.log_path + "rlw/"
        lib = self.lib_path + "rlw_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        PACKAGES_TXT = f"{repo}{self.slack_ver}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{self.slack_ver}/{md5_file}"
        ChangeLog_txt = f"{repo}{self.slack_ver}/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def alien(self):
        """Creating alien local library
        """
        ar = "x86"
        ver = self.slack_ver
        repo = self.def_repos_dict["alien"]
        log = self.log_path + "alien/"
        lib = self.lib_path + "alien_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = self.arch
        if self.meta.slack_rel == "current":
            ver = self.meta.slack_rel
        PACKAGES_TXT = f"{repo}/{ver}/{ar}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}/{ver}/{ar}/{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def slacky(self):
        """Creating slacky.eu local library
        """
        ar = ""
        repo = self.def_repos_dict["slacky"]
        log = self.log_path + "slacky/"
        lib = self.lib_path + "slacky_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "64"
        PACKAGES_TXT = f"{repo}slackware{ar}-{self.slack_ver}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}slackware{ar}-{self.slack_ver}/{md5_file}"

        ChangeLog_txt = f"{repo}slackware{ar}-{self.slack_ver}/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def conrad(self):
        """Creating slackers local library
        """
        repo = self.def_repos_dict["conrad"]
        log = self.log_path + "conrad/"
        lib = self.lib_path + "conrad_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        PACKAGES_TXT = f"{repo}{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def slonly(self):
        """Creating slackers local library
        """
        ar = f"{self.slack_ver}-x86"
        repo = self.def_repos_dict["slonly"]
        log = self.log_path + "slonly/"
        lib = self.lib_path + "slonly_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = f"{self.slack_ver}-x86_64"
        if self.meta.slack_rel == "current":
            ar = f"{self.meta.slack_rel}-x86"
        if self.meta.slack_rel == "current" and self.arch == "x86_64":
            ar = f"{self.meta.slack_rel}-x86_64"
        PACKAGES_TXT = f"{repo}{ar}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{ar}/{md5_file}"
        ChangeLog_txt = f"{repo}{ar}/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def ktown(self):
        """Creating alien ktown local library
        """
        repo = self.def_repos_dict["ktown"]
        log = self.log_path + "ktown/"
        lib = self.lib_path + "ktown_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        PACKAGES_TXT = f"{repo}{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def multi(self):
        """Creating alien multilib local library
        """
        ver = self.slack_ver
        repo = self.def_repos_dict["multi"]
        log = self.log_path + "multi/"
        lib = self.lib_path + "multi_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.meta.slack_rel == "current":
            ver = self.meta.slack_rel
        PACKAGES_TXT = f"{repo}{ver}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{ver}/{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def slacke(self):
        """Creating Slacke local library
        """
        ar = ""
        repo = self.def_repos_dict["slacke"]
        log = self.log_path + "slacke/"
        lib = self.lib_path + "slacke_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "64"
        version = self.meta.slacke_sub_repo[1:-1]
        PACKAGES_TXT = (f"{repo}slacke{version}/slackware{ar}-"
                        f"{self.slack_ver}/{lib_file}")
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = (f"{repo}slacke{version}/slackware{ar}-"
                         f"{self.slack_ver}/{md5_file}")
        ChangeLog_txt = (f"{repo}slacke{version}/slackware{ar}-"
                         f"{self.slack_ver}/{log_file}")
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def salix(self):
        """Creating SalixOS local library
        """
        ar = "i486"
        repo = self.def_repos_dict["salix"]
        log = self.log_path + "salix/"
        lib = self.lib_path + "salix_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "x86_64"
        PACKAGES_TXT = f"{repo}{ar}/{self.slack_ver}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{ar}/{self.slack_ver}/{md5_file}"
        ChangeLog_txt = f"{repo}{ar}/{self.slack_ver}/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def slackl(self):
        """Creating slackel.gr local library
        """
        ar = "i486"
        repo = self.def_repos_dict["slackl"]
        log = self.log_path + "slackl/"
        lib = self.lib_path + "slackl_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "x86_64"
        PACKAGES_TXT = f"{repo}{ar}/current/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{ar}/current/{md5_file}"
        ChangeLog_txt = f"{repo}{ar}/current/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def rested(self):
        """Creating alien restricted local library
        """
        repo = self.def_repos_dict["rested"]
        log = self.log_path + "rested/"
        lib = self.lib_path + "rested_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        PACKAGES_TXT = f"{repo}{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def msb(self):
        """Creating MATE local library
        """
        ar = "x86"
        ver_slack = self.slack_ver
        repo = self.def_repos_dict["msb"]
        log = self.log_path + "msb/"
        lib = self.lib_path + "msb_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "x86_64"
        version = self.meta.msb_sub_repo[1:-1]
        if self.meta.slack_rel == "current":
            ver_slack = self.meta.slack_rel
        PACKAGES_TXT = f"{repo}{ver_slack}/{version}/{ar}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{ver_slack}/{version}/{ar}/{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def csb(self):
        """Creating Cinnamon local library
        """
        ar = "x86"
        ver_slack = self.slack_ver
        repo = self.def_repos_dict["csb"]
        log = self.log_path + "csb/"
        lib = self.lib_path + "csb_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "x86_64"
        if self.meta.slack_rel == "current":
            ver_slack = self.meta.slack_rel
        PACKAGES_TXT = f"{repo}{ver_slack}/{ar}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{ver_slack}/{ar}/{md5_file}"
        ChangeLog_txt = f"{repo}{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def connos(self):
        """Creating connochaetos (slack-n-free) local library
        """
        nickname = "slack-n-free"
        ar = ""
        repo = self.def_repos_dict["connos"]
        log = self.log_path + "connos/"
        lib = self.lib_path + "connos_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "64"
        PACKAGES_TXT = f"{repo}{nickname}{ar}-{self.slack_ver}/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{nickname}{ar}-{self.slack_ver}/{md5_file}"
        ChangeLog_txt = f"{repo}{nickname}{ar}-{self.slack_ver}/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def mles(self):
        """Creating Microlinux local library
        """
        ar = "32"
        repo = self.def_repos_dict["mles"]
        log = self.log_path + "mles/"
        lib = self.lib_path + "mles_repo/"
        repo_name = log[:-1].split("/")[-1]
        lib_file = "PACKAGES.TXT"
        # lst_file = ""
        md5_file = "CHECKSUMS.md5"
        log_file = "ChangeLog.txt"
        self.make_dir([log, lib])
        if self.arch == "x86_64":
            ar = "64"
        version = self.meta.mles_sub_repo[1:-1]
        PACKAGES_TXT = f"{repo}{version}-{self.slack_ver}-{ar}bit/{lib_file}"
        FILELIST_TXT = ""
        CHECKSUMS_MD5 = f"{repo}{version}-{self.slack_ver}-{ar}bit/{md5_file}"
        ChangeLog_txt = f"{repo}{version}-{self.slack_ver}-{ar}bit/{log_file}"
        if self.check:
            return self.checks_logs(log, ChangeLog_txt)
        self.down(lib, PACKAGES_TXT, repo_name)
        self.down(lib, CHECKSUMS_MD5, repo_name)
        self.down(log, ChangeLog_txt, repo_name)
        self.remote(log, ChangeLog_txt, lib, PACKAGES_TXT, CHECKSUMS_MD5,
                    FILELIST_TXT, repo_name)

    def down(self, path, link, repo):
        """Download files
        """
        filename = link.split("/")[-1]
        if not os.path.isfile(path + filename):
            Download(path, link.split(), repo).start()

    def remote(self, *args):
        """Remove and recreate files
        """
        log_path = args[0]
        ChangeLog_txt = args[1]
        lib_path = args[2]
        PACKAGES_TXT = args[3]
        CHECKSUMS_MD5 = args[4]
        FILELIST_TXT = args[5]
        repo = args[6]

        if self.checks_logs(log_path, ChangeLog_txt):
            # remove old files
            self.file_remove(log_path, ChangeLog_txt.split("/")[-1])
            self.file_remove(lib_path, PACKAGES_TXT.split("/")[-1])
            self.file_remove(lib_path, CHECKSUMS_MD5.split("/")[-1])
            self.file_remove(lib_path, FILELIST_TXT.split("/")[-1])
            if repo == "slack":
                dirs = ["core/", "extra/"]
                for d in dirs:
                    self.file_remove(lib_path + d, "PACKAGES.TXT")
                    self.file_remove(lib_path + d, "CHECKSUMS.md5")
                self.down(lib_path + "core/", PACKAGES_TXT, repo)
                self.down(lib_path + "core/", CHECKSUMS_MD5, repo)
                self.down(lib_path + "extra/", self.EXTRA, repo)
                self.down(lib_path + "extra/", self.EXT_CHECKSUMS, repo)
            # download new files
            if repo != "slack":
                self.down(lib_path, PACKAGES_TXT, repo)
                self.down(lib_path, CHECKSUMS_MD5, repo)
            self.down(lib_path, FILELIST_TXT, repo)
            self.down(log_path, ChangeLog_txt, repo)

    def merge(self, path, outfile, infiles):
        """Merge files
        """
        code = "utf-8"
        with open(path + outfile, 'w', encoding=code) as out_f:
            for f in infiles:
                if os.path.isfile(f"{path}{f}"):
                    # checking the encoding before read the file
                    code = self.check_encoding(path, f)
                    with open(path + f, "r", encoding=code) as in_f:
                        for line in in_f:
                            out_f.write(line)

    def file_remove(self, path, filename):
        """Check if filename exists and remove
        """
        if os.path.isfile(path + filename):
            os.remove(path + filename)

    def checks_logs(self, log_path, url):
        """Checks ChangeLog.txt for changes
        """
        local = ""
        filename = url.split("/")[-1]
        server = FileSize(url).server()
        if os.path.isfile(log_path + filename):
            local = FileSize(log_path + filename).local()
        if server != local:
            return True
        return False


class Upgrade:

    def __init__(self):
        self.meta = _meta_
        self.log_path = self.meta.log_path
        self.lib_path = self.meta.lib_path

    def run(self, repos):
        """Removing and creating the packages lists
        """
        repositories = self.meta.repositories

        # Replace the enabled repositories from user defined
        if repos:
            repositories = repos

        for repo in repositories:
            changelogs = f"{self.log_path}{repo}/ChangeLog.txt"
            if os.path.isfile(changelogs):
                os.remove(changelogs)

            if os.path.isdir(f"{self.lib_path}{repo}_repo/"):
                for f in os.listdir(f"{self.lib_path}{repo}_repo/"):
                    files = f"{self.lib_path}{repo}_repo/{f}"
                    if os.path.isfile(files):
                        os.remove(files)
                    elif os.path.isdir(files):
                        shutil.rmtree(files)
        update = Update()
        update.run(repos)


class Update:

    def __init__(self):
        self.meta = _meta_
        self.grey = _meta_.color["GREY"]
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.cyan = _meta_.color["CYAN"]
        self.endc = _meta_.color["ENDC"]
        self.done = f"{self.green}Done{self.endc}\n"
        self.error = f"{self.red}Error{self.endc}\n"

    def run(self, repos):
        """Update repositories lists
        """
        print("\nCheck and update repositories:\n")
        default = self.meta.default_repositories
        enabled = self.meta.repositories
        custom = Repo().custom_repository()

        # Replace the enabled repositories from user defined
        if repos:
            enabled = repos

        for repo in enabled:
            if check_for_local_repos(repo) is True:
                continue
            self.done_msg(repo)
            if repo in default:
                getattr(Initialization(False), repo)()
                print(self.done, end="")
            elif repo in custom:
                Initialization(False).custom(repo)
                print(self.done, end="")
            else:
                print(self.error, end="")
        print()   # new line at end
        raise SystemExit()

    def done_msg(self, repo):
        print(f"{self.grey}Check repository "
              f"[{self.cyan}{repo}{self.grey}] ... "
              f"{self.endc}", end="", flush=True)


def check_exists_repositories(repo):
    """Checking if repositories exists by PACKAGES.TXT file
    """
    pkg_list = "PACKAGES.TXT"
    if repo == "sbo":
        pkg_list = "SLACKBUILDS.TXT"
    elif check_for_local_repos(repo) is True:
        pkg_list = "PACKAGES.TXT"
    if not os.path.isfile(f"{_meta_.lib_path}{repo}_repo/{pkg_list}"):
        return False


def check_for_local_repos(repo):
    """Check if repository is local
    """
    repos_dict = Repo().default_repository()
    if repo in repos_dict:
        repo_url = repos_dict[repo]
        if repo_url.startswith("file:///"):
            return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# upgrade_checklist.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.dialog_box import DialogUtil
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package
from slpkg.pkg.installed import GetFromInstalled


def choose_upg(packages):
    """Creating checklist to choose packages for upgrade
    """
    selected_packages, data = [], []
    if packages:
        for pkg in packages:
            name = GetFromInstalled(pkg).name()
            ver = GetFromInstalled(pkg).version()
            binary = f"{name}-{ver}"
            installed = find_package(binary + _meta_.sp, _meta_.pkg_path)[0]
            data.append(installed)
        text = "Press 'spacebar' to unchoose packages from upgrade"
        title = " Upgrade "
        backtitle = f"{_meta_.__all__} {_meta_.__version__}"
        status = True
        pkgs = DialogUtil(data, text, title, backtitle,
                          status).checklist()
        pkgs = [] if pkgs is None else pkgs
        for pkg in pkgs:
            name = split_package(pkg)[0]
            if name in packages:
                selected_packages.append(name)
        if not selected_packages:
            raise SystemExit(1)
        print()
    return selected_packages



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# slack_find.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
from pkg_resources import parse_version

from slpkg.messages import Msg
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package


def slack_package(prgnam):
    """Return maximum binary Slackware package from output directory
    """
    msg = Msg()
    TAG, binaries, cache, binary = "_SBo", [], "0", ""

    for exp in os.environ.keys():
        if exp == "TAG":
            TAG = os.environ["TAG"]
            break

    for pkg in find_package(prgnam, _meta_.output):
        if pkg.startswith(prgnam) and pkg[:-4].endswith(TAG):
            binaries.append(pkg)

    for bins in binaries:
        binary = bins
        if parse_version(bins) > parse_version(cache):
            binary = bins
            cache = binary

    if not binary:
        msg.pkg_not_found("\n", prgnam, "Can't build", "\n")
        raise SystemExit(1)

    return ["".join(_meta_.output + binary)]



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# slpkg_update.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import re
import tarfile
import subprocess

from slpkg.url_read import URL
from slpkg.checksum import check_md5
from slpkg.downloader import Download
from slpkg.grep_md5 import pkg_checksum
from slpkg.__metadata__ import MetaData as _meta_


def it_self_update():
    """Check from GitLab slpkg repository if new version is available
    download and update itself
    """
    __new_version__ = ""
    repository = "gitlab"
    branch = "master"
    ver_link = ("https://{0}.com/{1}/{2}/raw/"
                "{3}/{4}/__metadata__.py".format(repository,
                                                 _meta_.__author__,
                                                 _meta_.__all__,
                                                 branch,
                                                 _meta_.__all__))
    version_data = URL(ver_link).get_request()

    for line in version_data.splitlines():
        line = line.strip()
        if line.startswith("__version_info__"):
            __new_version__ = ".".join(re.findall(r"\d+", line))

    if __new_version__ > _meta_.__version__:
        if _meta_.default_answer in ["y", "Y"]:
            answer = _meta_.default_answer
        else:
            print("\nNew version '{0}-{1}' is available !\n".format(
                _meta_.__all__, __new_version__))
            try:
                answer = input("Would you like to upgrade [y/N]? ")
            except EOFError:
                raise SystemExit("\n")

        if answer in ["y", "Y"]:
            print()   # new line after answer
        else:
            raise SystemExit()

        dwn_link = ["https://{0}.com/{1}/{2}/-/archive/"
                    "{3}/{4}-{5}.tar.gz".format(repository,
                                                _meta_.__author__,
                                                _meta_.__all__,
                                                __new_version__,
                                                _meta_.__all__,
                                                __new_version__)]

        if not os.path.exists(_meta_.build_path):
            os.makedirs(_meta_.build_path)

        Download(_meta_.build_path, dwn_link, repo="").start()
        os.chdir(_meta_.build_path)
        slpkg_tar_file = f"slpkg-{__new_version__}.tar.gz"
        tar = tarfile.open(slpkg_tar_file)
        tar.extractall()
        tar.close()
        file_name = f"{_meta_.__all__}-{__new_version__}"
        os.chdir(file_name)
        check_md5(pkg_checksum(slpkg_tar_file, _meta_.__all__),
                  _meta_.build_path + slpkg_tar_file)
        subprocess.call("chmod +x {0}".format("install.sh"), shell=True)
        subprocess.call("sh install.sh", shell=True)

    else:
        print(f"\n{_meta_.__all__}: There is no new version, "
              "already you use the last!\n")
    raise SystemExit()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# greps.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.__metadata__ import MetaData as _meta_


class SBoGrep(Utils):
    """Grab data from SLACKBUILDS.TXT file
    """
    def __init__(self, name):
        self.name = name
        self.meta = _meta_
        arch64 = "x86_64"
        self.line_name = "SLACKBUILD NAME: "
        self.line_files = "SLACKBUILD FILES: "
        self.line_down = "SLACKBUILD DOWNLOAD: "
        self.line_down_64 = f"SLACKBUILD DOWNLOAD_{arch64}: "
        self.line_req = "SLACKBUILD REQUIRES: "
        self.line_ver = "SLACKBUILD VERSION: "
        self.line_md5 = "SLACKBUILD MD5SUM: "
        self.line_md5_64 = f"SLACKBUILD MD5SUM_{arch64}: "
        self.line_des = "SLACKBUILD SHORT DESCRIPTION: "
        self.sbo_txt = self.meta.lib_path + "sbo_repo/SLACKBUILDS.TXT"
        self.answer = ["y", "Y"]
        self.unst = ["UNSUPPORTED", "UNTESTED"]
        self.SLACKBUILDS_TXT = self.read_file(self.sbo_txt)

    def _names_grabbing(self):
        """Generator that collecting all packages names
        """
        for line in self.SLACKBUILDS_TXT.splitlines():
            if line.startswith(self.line_name):
                yield line[17:].strip()

    def names(self):
        """Alias method convert generator and return
        a list
        """
        return list(self._names_grabbing())

    def source(self):
        """Grab sources downloads links
        """
        source, source64, = "", ""
        for line in self.SLACKBUILDS_TXT.splitlines():
            if line.startswith(self.line_name):
                sbo_name = line[17:].strip()
            if line.startswith(self.line_down):
                if sbo_name == self.name and line[21:].strip():
                    source = line[21:]
            if line.startswith(self.line_down_64):
                if sbo_name == self.name and line[28:].strip():
                    source64 = line[28:]
        return self._select_source_arch(source, source64)

    def _select_source_arch(self, source, source64):
        """Return sources by arch
        """
        src = ""
        if self.meta.arch == "x86_64":
            if source64:
                src = source64
            else:
                src = source
            if self.meta.skip_unst in self.answer and source64 in self.unst:
                src = source
        else:
            if source:
                src = source
            if self.meta.skip_unst in self.answer and source in self.unst:
                src = source64
        return src

    def requires(self):
        """Grab package requirements
        """
        for line in self.SLACKBUILDS_TXT.splitlines():
            if line.startswith(self.line_name):
                sbo_name = line[17:].strip()
            if line.startswith(self.line_req):
                if sbo_name == self.name:
                    return line[21:].strip().split()

    def version(self):
        """Grab package version
        """
        for line in self.SLACKBUILDS_TXT.splitlines():
            if line.startswith(self.line_name):
                sbo_name = line[17:].strip()
            if line.startswith(self.line_ver):
                if sbo_name == self.name:
                    return line[20:].strip()

    def checksum(self):
        """Grab checksum string
        """
        md5sum, md5sum64, = [], []
        for line in self.SLACKBUILDS_TXT.splitlines():
            if line.startswith(self.line_name):
                sbo_name = line[17:].strip()
            if line.startswith(self.line_md5_64):
                if sbo_name == self.name and line[26:].strip():
                    md5sum64 = line[26:].strip().split()
            if line.startswith(self.line_md5):
                if sbo_name == self.name and line[19:].strip():
                    md5sum = line[19:].strip().split()
        return self._select_md5sum_arch(md5sum, md5sum64)

    def _select_md5sum_arch(self, md5sum, md5sum64):
        """Return checksums by arch
        """
        if md5sum and md5sum64:
            if self.meta.arch == "x86_64":
                return md5sum64
            else:
                return md5sum
        if md5sum:
            return md5sum
        else:
            return md5sum64

    def description(self):
        """Grab package version
        """
        for line in self.SLACKBUILDS_TXT.splitlines():
            if line.startswith(self.line_name):
                sbo_name = line[17:].strip()
            if line.startswith(self.line_des):
                if sbo_name == self.name:
                    return line[31:].strip()

    def files(self):
        """Grab files
        """
        for line in self.SLACKBUILDS_TXT.splitlines():
            if line.startswith(self.line_name):
                sbo_name = line[17:].strip()
            if line.startswith(self.line_files):
                if sbo_name == self.name:
                    return line[18:].strip()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# check.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from pkg_resources import parse_version

from slpkg.messages import Msg
from slpkg.splitting import split_package
from slpkg.upgrade_checklist import choose_upg
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package

from slpkg.binary.greps import repo_data
from slpkg.binary.repo_init import RepoInit


def pkg_upgrade(repo, skip, flag):
    """Checking packages for upgrade
    """
    msg = Msg()
    msg.checking()
    PACKAGES_TXT = RepoInit(repo).fetch()[0]
    pkgs_for_upgrade = []
    # name = data[0]
    # location = data[1]
    # size = data[2]
    # unsize = data[3]
    data = repo_data(PACKAGES_TXT, repo, flag="")
    for pkg in installed():
        inst_pkg = split_package(pkg)
        for name in data[0]:
            if name:    # this tips because some pkg_name is empty
                repo_pkg = split_package(name[:-4])
            if (repo_pkg[0] == inst_pkg[0] and
                parse_version(repo_pkg[1]) > parse_version(inst_pkg[1]) and
                repo_pkg[3] >= inst_pkg[3] and
                    inst_pkg[0] not in skip and
                    repo_pkg[1] != "blacklist"):
                pkgs_for_upgrade.append(repo_pkg[0])
    msg.done()
    if "--checklist" in flag:
        pkgs_for_upgrade = choose_upg(pkgs_for_upgrade)
    return pkgs_for_upgrade


def installed():
    """Return all installed packages
    """
    return find_package("", _meta_.pkg_path)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# downloader.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import tarfile
import subprocess

from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.slack.slack_version import slack_ver
from slpkg.__metadata__ import MetaData as _meta_


class Download(Utils):
    """Downloader manager. Slpkg use wget by default but support
    curl, aria2 and httpie
    """
    def __init__(self, path, url, repo):
        self.path = path
        self.url = url
        self.repo = repo
        self.file_name = ""
        self.meta = _meta_
        self.green = _meta_.color["GREEN"]
        self.red = _meta_.color["RED"]
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.dir_prefix = ""
        self.downder = self.meta.downder
        self.downder_options = self.meta.downder_options

    def start(self):
        """Download files using wget or other downloader.
        Optional curl, aria2c and httpie
        """
        dwn_count = 1
        self._directory_prefix()
        for dwn in self.url:
            self.file_name = self.fix_file_name(dwn.split("/")[-1])

            if dwn.startswith("file:///"):
                source_dir = dwn[7:-7].replace(slack_ver(), "")
                self._make_tarfile(self.file_name, source_dir)

            self._check_certificate()
            print(f"\n[{dwn_count}/{len(self.url)}][ {self.green}"
                  f"Download{self.endc} ] --> {self.file_name}\n")
            if self.downder in ["wget"]:
                subprocess.call(f"{self.downder} {self.downder_options}"
                                f" {self.dir_prefix}{self.path} {dwn}",
                                shell=True)
            if self.downder in ["aria2c"]:
                subprocess.call(f"{self.downder} {self.downder_options}"
                                f" {self.dir_prefix}{self.path[:-1]} {dwn}",
                                shell=True)
            elif self.downder in ["curl", "http"]:
                subprocess.call(f"{self.downder} {self.downder_options}"
                                f" {self.path}{self.file_name} {dwn}",
                                shell=True)
            self._check_if_downloaded()
            dwn_count += 1

    def _make_tarfile(self, output_filename, source_dir):
        """Create .tar.gz file
        """
        with tarfile.open(output_filename, "w:gz") as tar:
            tar.add(source_dir, arcname=os.path.basename(source_dir))

    def _directory_prefix(self):
        """Downloader options for specific directory
        """
        if self.downder == "wget":
            self.dir_prefix = "--directory-prefix="
        elif self.downder == "aria2c":
            self.dir_prefix = "--dir="

    def _check_if_downloaded(self):
        """Check if file downloaded
        """
        if not os.path.isfile(self.path + self.file_name):
            print()
            self.msg.template(78)
            print(f"| Download '{self.file_name}' file"
                  f" [ {self.red}FAILED{self.endc} ]")
            self.msg.template(78)
            print()
            if not self.msg.answer() in ["y", "Y"]:
                raise SystemExit()

    def _check_certificate(self):
        """Check for certificates options for wget
        """
        if (self.file_name.startswith("jdk-") and self.repo == "sbo" and
                self.downder == "wget"):
            certificate = (' --no-check-certificate --header="Cookie: '
                           'oraclelicense=accept-securebackup-cookie"')
            self.msg.template(78)
            print(f"| '{certificate[:23].strip()}' need to go"
                  f" ahead downloading")
            self.msg.template(78)
            print()
            self.downder_options += certificate
            if not self.msg.answer() in ["y", "Y"]:
                raise SystemExit()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# repoenable.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.dialog_box import DialogUtil
from slpkg.__metadata__ import MetaData as _meta_


class RepoEnable(Utils):
    """Read repositories.conf file and update with new enabled or
    disabled repositories
    """
    def __init__(self):
        self.meta = _meta_
        self.red = _meta_.color["RED"]
        self.grey = _meta_.color["GREY"]
        self.endc = _meta_.color["ENDC"]
        self.msg = Msg()
        self.tag = "[REPOSITORIES]"
        self.tag_line = False
        self.repositories_conf = "repositories.conf"
        self.conf = self.read_file(f"{self.meta.conf_path}"
                                   f"{self.repositories_conf}")
        self.enabled = []
        self.disabled = []
        self.selected = []

    def choose(self):
        """Choose repositories
        """
        keys = """
Choose repositories at the right side for enable or to the
left side for disable.

Keys: SPACE   select or deselect the highlighted repositories,
             move it between the left and right lists
          ^       move the focus to the left list
          $       move the focus to the right list
        TAB     move focus
      ENTER   press the focused button

      Disabled <---------------- REPOSITORIES ----------------> Enabled"""
        self.read_enabled()
        self.read_disabled()
        text, title, backtitle, status = keys, " Repositories ", "", False
        self.selected = DialogUtil(self.disabled, text, title, backtitle,
                                   status).buildlist(self.enabled)
        if self.selected is not None:
            self.update_repos()
        else:
            self.selected = self.enabled
        self.clear_screen()
        self.reference()

    def read_enabled(self):
        """Read enable repositories
        """
        for line in self.conf.splitlines():
            line = line.lstrip()
            if self.tag in line:
                self.tag_line = True
            if (line and self.tag_line and not line.startswith("#") and
                    self.tag not in line):
                self.enabled.append(line)
        self.tag_line = False

    def read_disabled(self):
        """Read disable repositories
        """
        for line in self.conf.splitlines():
            line = line.lstrip()
            if self.tag in line:
                self.tag_line = True
            if self.tag_line and line.startswith("#"):
                line = "".join(line.split("#")).strip()
                self.disabled.append(line)
        self.tag_line = False

    def update_repos(self):
        """Update repositories.conf file with enabled or disabled
        repositories
        """
        with open(f"{self.meta.conf_path}"
                  f"{self.repositories_conf}", "w") as new_conf:
            for line in self.conf.splitlines():
                line = line.lstrip()
                if self.tag in line:
                    self.tag_line = True
                if self.tag_line and line.startswith("#"):
                    repo = "".join(line.split("#")).strip()
                    if repo in self.selected:
                        new_conf.write(line.replace(line, f"{repo}\n"))
                        continue
                if (self.tag_line and not line.startswith("#") and
                        line != self.tag):
                    repo = line.strip()
                    if repo not in self.selected:
                        new_conf.write(line.replace(line, f"# {line}\n"))
                        continue
                new_conf.write(f"{line}\n")

    def clear_screen(self):
        """Clear screen
        """
        os.system("clear")

    def reference(self):
        """Reference enable repositories
        """
        total_enabled = ", ".join(self.selected)
        if len(total_enabled) < 1:
            total_enabled = (f"{self.red}Are you crazy? This is a package "
                             f"manager for packages :p{self.endc}")
        self.msg.template(78)
        print("| Enabled repositories:")
        self.msg.template(78)
        print(f"| {total_enabled}")
        self.msg.template(78)
        print(f"{self.grey}Total {len(self.selected)}/"
              f"{len(self.enabled + self.disabled)} "
              f"repositories enabled.{self.endc}\n")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# log_deps.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package


def write_deps(deps_dict):
    """Write dependencies in a log file
    into directory `/var/log/slpkg/dep/`
    """
    for name, dependencies in deps_dict.items():
        if find_package(f"{name}-", _meta_.pkg_path):
            dep_path = f"{_meta_.log_path}dep/"
            if not os.path.exists(dep_path):
                os.mkdir(dep_path)
            if os.path.isfile(f"{dep_path}{name}"):
                os.remove(f"{dep_path}{name}")
            if len(dependencies) >= 1:
                with open(f"{dep_path}{name}", "w") as f:
                    for dep in dependencies:
                        f.write(f"{dep}\n")


#!/usr/bin/python3
# -*- coding: utf-8 -*-

# slackware_repo.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from slpkg.utils import Utils
from slpkg.binary.greps import repo_data
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_


def slackware_repository():
    """Return all official Slackware packages
    """
    utils = Utils()
    slack_repo, packages, names = [], [], []
    slack_repo = repo_data(
        utils.read_file(f"{_meta_.lib_path}slack_repo/PACKAGES.TXT"),
        "slack", "")
    for pkg in slack_repo[0]:
        names.append(split_package(pkg)[0])
        packages.append(pkg[:-4])
    return packages, names



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# status_deps.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os

from slpkg.utils import Utils
from slpkg.messages import Msg
from slpkg.graph import Graph
from slpkg.splitting import split_package
from slpkg.__metadata__ import MetaData as _meta_

from slpkg.pkg.find import find_package


class DependenciesStatus(Utils):
    """Print dependencies status used by packages
    """
    def __init__(self, image):
        self.image = image
        self.meta = _meta_
        self.msg = Msg()
        self.grey = self.meta.color["GREY"]
        self.green = self.meta.color["GREEN"]
        self.endc = self.meta.color["ENDC"]
        self.dmap = {}
        self.count_pkg = 0
        self.count_dep = 0
        self.dep_path = self.meta.log_path + "dep/"
        self.logs = find_package("", self.dep_path)
        if not self.logs:
            self.no_logs()
        self.installed = find_package("", self.meta.pkg_path)

    def data(self):
        """Check all installed packages and create
        dictionary database
        """
        for pkg in self.installed:
            if os.path.isfile(f"{self.meta.pkg_path}{pkg}"):
                name = split_package(pkg)[0]
                for log in self.logs:
                    deps = self.read_file(f"{self.dep_path}{log}")
                    for dep in deps.splitlines():
                        if name == dep:
                            if name not in self.dmap.keys():
                                self.dmap[name] = [log]
                                if not self.count_pkg:
                                    self.count_pkg = 1
                            else:
                                self.dmap[name] += [log]
        self.count_packages()

    def count_packages(self):
        """Count dependencies and packages
        """
        packages = []
        for pkg in self.dmap.values():
            packages += pkg
            self.count_dep += 1
        self.count_pkg = len(set(packages))

    def show(self):
        """Show dependencies status
        """
        self.data()
        print()
        self.msg.template(78)
        print(f"| Dependencies{' ' * 20}Packages")
        self.msg.template(78)
        for key, value in self.dmap.items():
            print("  {0}{1}{2}{3}{4}".format(
                self.green, key, self.endc, " " * (32-len(key)),
                ", ".join(value)))
        self.summary()
        if self.image:
            Graph(self.image).dependencies(self.dmap)

    def tree(self):
        """Like tree view mode
        """
        self.msg.template(78)
        print("| Dependencies\n"
              "| -- Packages")
        self.msg.template(78)
        self.data()
        for pkg, dep in self.dmap.items():
            print(f"+ {self.green}{pkg}{self.endc}")
            print("|")
            for d in dep:
                print(f"+-- {d}")
                print("|")
            print("\x1b[1A \n", end="", flush=True)
        self.summary()
        if self.image:
            Graph(self.image).dependencies(self.dmap)

    def no_logs(self):
        """Print message if no logs found
        """
        raise SystemExit("\n  There were no logs files. "
                         "Obviously not used the\n"
                         "  installation method with the command:\n"
                         "  '$ slpkg -s <repository> <packages>' yet.\n")

    def summary(self):
        """Summary by packages and dependencies
        """
        print("\nStatus summary")
        print("=" * 79)
        print("{0}found {1} dependencies in {2} packages.{3}\n".format(
            self.grey, self.count_dep, self.count_pkg, self.endc))



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# remove.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os
import shutil


from slpkg.__metadata__ import MetaData as _meta_


def delete_package(path, packages):
    """Delete downloaded packages
    """
    if _meta_.del_all in ["on", "ON"]:
        for pkg in packages:
            os.remove(f"{path}{pkg}")


def delete_folder(folder):
    """Delete folder with all files.
    """
    if _meta_.del_folder in ["on", "ON"] and os.path.exists(folder):
        shutil.rmtree(folder)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# __metadata__.py file is part of slpkg.

# Copyright 2014-2022 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
# All rights reserved.

# Slpkg is a user-friendly package manager for Slackware installations

# https://gitlab.com/dslackw/slpkg

# Slpkg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import os


# Slackware distribution branches
STABLE = "stable"
CURRENT = "current"


def remove_repositories(repositories, default_repositories):
    """
    Remove no default repositories
    """
    for repo in repositories:
        if repo in default_repositories:
            yield repo


def update_repositories(repositories, conf_path):
    """
    Upadate with user custom repositories
    """
    repo_file = f"{conf_path}custom-repositories"
    if os.path.isfile(repo_file):
        f = open(repo_file, "r")
        repositories_list = f.read()
        f.close()
        for line in repositories_list.splitlines():
            line = line.lstrip()
            if line and not line.startswith("#"):
                repositories.append(line.split()[0])
    return repositories


def grab_sub_repo(repositories, repos):
    """
    Grab SUB_REPOSITORY
    """
    for i, repo in enumerate(repositories):
        if repos in repo:
            sub = repositories[i].replace(repos, "")
            repositories[i] = repos
            return sub
    return ""


def select_slack_release(slack_rel):
    """
    Warning message if Slackware release not defined or
    defined wrong
    """
    if slack_rel not in [STABLE, CURRENT]:
        return "FAULT"
    return slack_rel


class MetaData:

    __all__ = "slpkg"
    __author__ = "dslackw"
    __version_info__ = (3, 9, 8)
    __version__ = "{0}.{1}.{2}".format(*__version_info__)
    __license__ = "GNU General Public License v3 (GPLv3)"
    __email__ = "d.zlatanidis@gmail.com"
    __maintainer__ = "Dimitris Zlatanidis (dslackw)"
    __homepage__ = "https://dslackw.gitlab.io/slpkg/"

    # Default Slackware release
    slack_rel = STABLE

    # Configuration path
    conf_path = f"/etc/{__all__}/"

    # tmp paths
    tmp = "/tmp/"
    tmp_path = f"{tmp}{__all__}/"

    # Default configuration values
    _conf_slpkg = {
        "RELEASE": STABLE,
        "SLACKWARE_VERSION": "off",
        "COMP_ARCH": "off",
        "REPOSITORIES": ["slack", "sbo", "rlw", "alien",
                         "slacky", "conrad", "slonly",
                         "ktown{latest}", "multi", "slacke{18}",
                         "salix", "slackl", "rested", "msb{1.18}",
                         "csb", "connos", "mles{desktop}"],
        "BUILD_PATH": "/tmp/slpkg/build/",
        "SBOSRCARCH": "off",
        "SBOSRCARCH_LINK": "http://slackware.uk/sbosrcarch/by-name/",
        "PACKAGES": "/tmp/slpkg/packages/",
        "PATCHES": "/tmp/slpkg/patches/",
        "CHECKMD5": "on",
        "DEL_ALL": "on",
        "DEL_BUILD": "off",
        "SBO_BUILD_LOG": "on",
        "MAKEFLAGS": "off",
        "DEFAULT_ANSWER": "n",
        "REMOVE_DEPS_ANSWER": "n",
        "SKIP_UNST": "n",
        "RSL_DEPS": "on",
        "DEL_DEPS": "off",
        "USE_COLORS": "on",
        "DOWNDER": "wget",
        "DOWNDER_OPTIONS": "-c -N",
        "SLACKPKG_LOG": "on",
        "ONLY_INSTALLED": "off",
        "EDITOR": "nano",
        "NOT_DOWNGRADE": "off",
        "HTTP_PROXY": "",
    }

    default_repositories = ["slack", "sbo", "rlw", "alien", "slacky", "conrad",
                            "slonly", "ktown", "multi", "slacke", "salix",
                            "slackl", "rested", "msb", "csb", "connos", "mles"]

    # read value from configuration file
    repositories = []
    for files in ["slpkg.conf", "repositories.conf"]:
        if os.path.isfile(f"{conf_path}{files}"):
            f = open(f"{conf_path}{files}", "r")
            conf = f.read()
            f.close()
            for line in conf.splitlines():
                line = line.lstrip()
                if line and not line.startswith("#"):
                    if files == "slpkg.conf":
                        _conf_slpkg[line.split("=")[0]] = line.split("=")[1]
                    elif files == "repositories.conf":
                        repositories.append(line)

    # Set values from configuration file
    slack_rel = _conf_slpkg["RELEASE"]
    slackware_version = _conf_slpkg["SLACKWARE_VERSION"]
    comp_arch = _conf_slpkg["COMP_ARCH"]
    build_path = _conf_slpkg["BUILD_PATH"]
    sbosrcarch = _conf_slpkg["SBOSRCARCH"]
    sbosrcarch_link = _conf_slpkg["SBOSRCARCH_LINK"]
    slpkg_tmp_packages = _conf_slpkg["PACKAGES"]
    slpkg_tmp_patches = _conf_slpkg["PATCHES"]
    checkmd5 = _conf_slpkg["CHECKMD5"]
    del_all = _conf_slpkg["DEL_ALL"]
    del_folder = _conf_slpkg["DEL_BUILD"]
    sbo_build_log = _conf_slpkg["SBO_BUILD_LOG"]
    makeflags = _conf_slpkg["MAKEFLAGS"]
    default_answer = _conf_slpkg["DEFAULT_ANSWER"]
    remove_deps_answer = _conf_slpkg["REMOVE_DEPS_ANSWER"]
    skip_unst = _conf_slpkg["SKIP_UNST"]
    rsl_deps = _conf_slpkg["RSL_DEPS"]
    del_deps = _conf_slpkg["DEL_DEPS"]
    use_colors = _conf_slpkg["USE_COLORS"]
    downder = _conf_slpkg["DOWNDER"]
    downder_options = _conf_slpkg["DOWNDER_OPTIONS"]
    slackpkg_log = _conf_slpkg["SLACKPKG_LOG"]
    only_installed = _conf_slpkg["ONLY_INSTALLED"]
    editor = _conf_slpkg["EDITOR"]
    not_downgrade = _conf_slpkg["NOT_DOWNGRADE"]
    http_proxy = _conf_slpkg["HTTP_PROXY"]

    # SBo downloading sources path
    SBo_SOURCES = build_path + "_SOURCES/"

    # Remove any gaps
    repositories = [repo.strip() for repo in repositories]

    # Check Slackware release
    slack_rel = select_slack_release(slack_rel)

    # Grap sub repositories
    ktown_kde_repo = grab_sub_repo(repositories, "ktown")
    slacke_sub_repo = grab_sub_repo(repositories, "slacke")
    msb_sub_repo = grab_sub_repo(repositories, "msb")
    mles_sub_repo = grab_sub_repo(repositories, "mles")

    # remove no default repositories
    repositories = list(remove_repositories(repositories,
                                            default_repositories))
    # add custom repositories
    update_repositories(repositories, conf_path)

    color = {
        "RED": "\x1b[31m",
        "GREEN": "\x1b[32m",
        "YELLOW": "\x1b[33m",
        "CYAN": "\x1b[36m",
        "GREY": "\x1b[38;5;247m",
        "ENDC": "\x1b[0m"
    }

    if use_colors in ["off", "OFF"]:
        color = {
            "RED": "",
            "GREEN": "",
            "YELLOW": "",
            "CYAN": "",
            "GREY": "",
            "ENDC": ""
        }

    CHECKSUMS_link = (f"https://gitlab.com/{__author__}/{__all__}/raw/"
                      "master/CHECKSUMS.md5")

    # current path
    try:
        path = os.getcwd() + "/"
    except OSError:
        path = tmp_path

    # library path
    lib_path = "/var/lib/slpkg/"

    # log path
    log_path = "/var/log/slpkg/"

    # packages log files path
    pkg_path = "/var/log/packages/"

    # slackpkg lib path
    slackpkg_lib_path = "/var/lib/slackpkg/"

    # computer architecture
    if comp_arch in ["off", "OFF"]:
        arch = os.uname()[4]
    else:
        arch = comp_arch

    # get sbo OUTPUT enviroment variable
    try:
        output = os.environ["OUTPUT"]
    except KeyError:
        output = tmp
    if not output.endswith("/"):
        output += "/"
